#========================================================================================
# package      DE_EPAGES::AmazonPayments::API::DTO::OrderReferenceInfo
# state        private
#========================================================================================
package DE_EPAGES::AmazonPayments::API::DTO::OrderReferenceInfo;

use DE_EPAGES::AmazonPayments::API::Constants qw ( AMAZON_ORDER_STATUS ORDER_DRAFT ORDER_SUSPENDED ORDER_OPEN ORDER_CLOSED );
use DE_EPAGES::AmazonPayments::API::DTO::AddressInfo;
use DE_EPAGES::Core::API::Error qw ( Error );
use strict;

use metaclass (
    metaclass   => 'Moose::Meta::Class',
    error_class => 'DE_EPAGES::Core::API::MooseError',
);
use Moose;
use Moose::Util::TypeConstraints;

has id            => (
    is  => 'ro',
    isa => 'Str'
);

# status is the external status. Use transStatus to get the internal order status
has status        => (
    is  => 'ro',
    isa => enum [ values %{AMAZON_ORDER_STATUS()} ]
);

has address       => (
    is      => 'ro',
    isa     => 'Maybe[DE_EPAGES::AmazonPayments::API::DTO::AddressInfo]'
);

has createdOn     => (
    is => 'ro',
    isa => 'Str'
);

has buyerEmail     => (
    is => 'ro',
    isa => 'Maybe[Str]'
);

has constraints   => (
    is => 'ro',
    isa => 'ArrayRef[Str]',
    default => sub { [] },
    traits  => ['Array']
);

around BUILDARGS => sub {
      my $orig  = shift;
      my $class = shift;
      my ($hArgs) = @_;

      if ( $hArgs->{'status'}) {
          $hArgs->{'status'} = $hArgs->{'status'}->{'state'};
      }
      if ($hArgs->{'address'}) {
          $hArgs->{'address'} = DE_EPAGES::AmazonPayments::API::DTO::AddressInfo->new($hArgs->{'address'});
      }
      return $class->$orig($hArgs);
  };

#========================================================================================
# function     transStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TransStatus = $OrderDetails->transStatus;
#----------------------------------------------------------------------------------------
# description  Transforms the value of <function status> into a value that
#               can be used for the attribute LineItemPaymentAmazonPayments.TransStatus.
#----------------------------------------------------------------------------------------
# return       $TransStatus | line item transaction status | string
#========================================================================================
sub transStatus {
    my $self = shift;

    my $Status = $self->status;
    my $hStatusMap = { reverse %{AMAZON_ORDER_STATUS()} };
    Error('InvalidOrderStatus', { 'Status' => $Status }) unless exists $hStatusMap->{$Status};

    return $hStatusMap->{$Status};
}

#========================================================================================
# function     isInDraftStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsDraftStatus = $OrderDetails->isInDraftStatus();
#----------------------------------------------------------------------------------------
# description  Evaluates if order reference is in "Draft" status.
#----------------------------------------------------------------------------------------
# return       $IsDraftStatus | order reference has draft status | boolean
#========================================================================================
sub isInDraftStatus {
    my $self = shift;

    return $self->transStatus() eq ORDER_DRAFT;
}

#========================================================================================
# function     isInSuspendedStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsSuspendedStatus = $OrderDetails->isInSuspendedStatus();
#----------------------------------------------------------------------------------------
# description  Evaluates if order reference is in "Suspended" status.
#----------------------------------------------------------------------------------------
# return       $IsSuspendedStatus | order reference has suspended status | boolean
#========================================================================================
sub isInSuspendedStatus {
    my $self = shift;

    return $self->transStatus() eq ORDER_SUSPENDED;
}

#========================================================================================
# function     isInOpenStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsOpenStatus = $OrderDetails->isInOpenStatus();
#----------------------------------------------------------------------------------------
# description  Evaluates if order reference is in "Open" status.
#----------------------------------------------------------------------------------------
# return       $IsOpenStatus | order reference has open status | boolean
#========================================================================================
sub isInOpenStatus {
    my $self = shift;

    return $self->transStatus() eq ORDER_OPEN;
}

#========================================================================================
# function     isInClosedStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsClosedStatus = $OrderDetails->isInClosedStatus();
#----------------------------------------------------------------------------------------
# description  Evaluates if order reference is in "Closed" status.
#----------------------------------------------------------------------------------------
# return       $IsClosedStatus | order reference has closed status | boolean
#========================================================================================
sub isInClosedStatus {
    my $self = shift;

    return $self->transStatus() eq ORDER_CLOSED;
}

no Moose;
__PACKAGE__->meta->make_immutable;

1;