#========================================================================================
# package      DE_EPAGES::AmazonPayments::API::Object::AmazonRestClient
# base         DE_EPAGES::EPagesJ::API::Object::RestClient
# state        public
# description  Client for Amazon Payments
#========================================================================================
package DE_EPAGES::AmazonPayments::API::Object::AmazonRestClient;
use base DE_EPAGES::EPagesJ::API::Object::RestClient;

use strict;
use DE_EPAGES::AmazonPayments::API::Constants qw ( AMAZON_LIVE_MODE AMAZON_SANDBOX_MODE );
use DE_EPAGES::AmazonPayments::API::DTO::AuthorizationDetails;
use DE_EPAGES::AmazonPayments::API::DTO::CaptureDetails;
use DE_EPAGES::AmazonPayments::API::DTO::OrderReferenceInfo;
use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestString );
use DE_EPAGES::EPagesJ::API::UriTemplateFactory qw ( CreateUriTemplates );
use Readonly;

Readonly my $CHECK_PATH => 'payments/amazon/check';
Readonly my $ORDER_PATH => 'payments/amazon/orders/{refId}';
Readonly my $ORDER_CONFIRMATION_PATH => 'payments/amazon/orders/{refId}/confirmation';
Readonly my $CLOSE_ORDER_PATH => 'payments/amazon/orders/{refId}/close';
Readonly my $CANCEL_ORDER_PATH => 'payments/amazon/orders/{refId}/cancel';

Readonly my $AUTHORIZE_PATH => 'payments/amazon/authorizations';
Readonly my $AUTHORIZATION_PATH => 'payments/amazon/authorizations/{authId}';
Readonly my $CLOSE_AUTHORIZATION_PATH => 'payments/amazon/authorizations/{authId}/close';

Readonly my $CAPTURE_PATH => 'payments/amazon/capture';
Readonly my $CAPTURE_DETAILS_PATH => 'payments/amazon/capture/{captureId}';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::AmazonPayments::API::Object::AmazonRestClient->new( %args );
#----------------------------------------------------------------------------------------
# description  Webservice client constructor
#----------------------------------------------------------------------------------------
# input        %args | Constructor arguments. Valid keys are:
#               <ul>
#                 <li>OperationMode - operation mode of the Amazon Payments method (LIVE or SANDBOX) - string</li>
#                 <li>Region - the region identifier to use - string</li>
#                 <li>hKeySet - Set of credentials
#                   <ul>
#                     <li>AccessKeyId - the Amazon access key - string</li>
#                     <li>SecrectKey - the secret key - string</li>
#                     <li>MerchantID - the merchant id - string</li>
#                   </ul> - ref.hash
#                 </li>
#                 <li>refId - Amazon order reference id (optional) - string</li>
#               </ul> | ref.hash
# return       $Object | new object | DE_EPAGES::AmazonPayments::API::Object::AmazonRestClient
#========================================================================================
sub new {
    my $class = shift;
    my %args  = @_;

    foreach my $Mandatory (qw(OperationMode Region )) {
        TestString($Mandatory, $args{$Mandatory});
    }
    foreach my $Mandatory(qw(CheckUrl OrderUrl)) {
        TestObject($Mandatory, $args{$Mandatory});
    }

    return __PACKAGE__->SUPER::new(%args);
}

#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RestClient = DE_EPAGES::AmazonPayments::API::Object::AmazonRestClient->create($PaymentMethod, $hArgs);
#----------------------------------------------------------------------------------------
# description  Factory method for a AmazonRestClient object.
#----------------------------------------------------------------------------------------
# input        $PaymentMethod | This context object will be used for initializing
#                   BaseURL and Resource parameters for the rest client.
#                   | DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments
# input        $hArgs | Additional arguments for constructor
#               <ul>
#                 <li>BaseURL  - the base url used for CreateUriTemplates - string</li>
#                 <li>Region - the region identifier to use - string</li>
#                 <li>hKeySet - used keys: AccessKeyId | access key | string<br/>
#                                   SecretKey | secret key | string<br/>
#                                  MerchantID | Amazon seller ID | string<br/> - ref.hash</li>
#                 <li>refId - Order reference id (optional) - string</li>
#               </ul> | ref.hash
# return       $RestClient | Client object | DE_EPAGES::AmazonPayments::API::Object::AmazonRestClient
#========================================================================================
sub create {
    my $class = shift;
    my ($PaymentMethod, $hArgs) = @_;

    my $hUriTemplates = CreateUriTemplates($PaymentMethod, [
            $CHECK_PATH,
            $ORDER_PATH,
            $ORDER_CONFIRMATION_PATH,
            $CLOSE_ORDER_PATH,
            $CANCEL_ORDER_PATH,
            $AUTHORIZE_PATH,
            $AUTHORIZATION_PATH,
            $CLOSE_AUTHORIZATION_PATH,
            $CAPTURE_PATH,
            $CAPTURE_DETAILS_PATH,
        ],
        $hArgs->{BaseURL}
    );

    $hArgs->{Region}   //= $PaymentMethod->get('AmazonRegion');
    $hArgs->{OperationMode} //= $PaymentMethod->get('IsInTestMode') ? AMAZON_SANDBOX_MODE : AMAZON_LIVE_MODE;
    $hArgs->{hKeySet}  //= $PaymentMethod->get(['MerchantID', 'AccessKeyId', 'SecretKey']);
    $hArgs->{CheckUrl} //= $hUriTemplates->{$CHECK_PATH}->uri;
    $hArgs->{OrderUrl} //= $hUriTemplates->{$ORDER_PATH};
    $hArgs->{OrderConfirmationUrl} //= $hUriTemplates->{$ORDER_CONFIRMATION_PATH};
    $hArgs->{CloseOrderUrl} //= $hUriTemplates->{$CLOSE_ORDER_PATH};
    $hArgs->{CancelOrderUrl} //= $hUriTemplates->{$CANCEL_ORDER_PATH};
    $hArgs->{AuthorizeUrl} //= $hUriTemplates->{$AUTHORIZE_PATH}->uri;
    $hArgs->{AuthorizationUrl} //= $hUriTemplates->{$AUTHORIZATION_PATH};
    $hArgs->{CloseAuthorizationUrl} //= $hUriTemplates->{$CLOSE_AUTHORIZATION_PATH};
    $hArgs->{CaptureUrl} //= $hUriTemplates->{$CAPTURE_PATH}->uri;
    $hArgs->{CaptureDetailsUrl} //= $hUriTemplates->{$CAPTURE_DETAILS_PATH};
    $hArgs->{isLogging} //= $PaymentMethod->get('PaymentType')->get('Logging');
    $hArgs->{LogCategory} //= 'AmazonPayments';

    return $class->SUPER::create($PaymentMethod, $hArgs);
}

#========================================================================================
# function     check
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Status = $Client->check();
#----------------------------------------------------------------------------------------
# description  Performs a REST call to the EpagesJ payment/amazon/check resource to
#               validate payment credentials
#----------------------------------------------------------------------------------------
# return       $Status | The check status returned by EPagesJ:
#                         OK, UNKNOWN_MERCHANT_ID or UNAUTHORIZED_DEVELOPER | string
#========================================================================================
sub check {
    my $self = shift;

    my $Url = $self->_setQueryArgs($self->_checkUrl);
    my $Response = $self->client->get($Url, $self->_httpHeaders());
    my $Content = $self->decodeResponse($Response->content);
    return $Content->{'status'};
}

#========================================================================================
# function     getOrderDetails
# state        public
#----------------------------------------------------------------------------------------
# syntax       $OrderReferenceInfo = $Client->getOrderDetails($refId);
#----------------------------------------------------------------------------------------
# description  Get detailed information about an Amazon Order Reference
#----------------------------------------------------------------------------------------
# input        $refId | Order reference id | string
# return       $OrderReferenceInfo | Order reference information | DE_EPAGES::AmazonPayments::API::DTO::OrderReferenceInfo
#========================================================================================
sub getOrderDetails {
    my $self = shift;
    my ($refId) = @_;

    my $Url = $self->_setQueryArgs($self->_orderUrl($refId));
    my $Response = $self->client->get($Url, $self->_httpHeaders());
    return DE_EPAGES::AmazonPayments::API::DTO::OrderReferenceInfo->new($self->decodeResponse($Response->content || '{}'));
}

#========================================================================================
# function     setOrderDetails
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Status = $Client->setOrderDetails($refId, $hArgs);
#----------------------------------------------------------------------------------------
# description  Set details on an Amazon Order Reference
#----------------------------------------------------------------------------------------
# input        $refId | Order reference id | string
# input        $hArgs | Detail values
#                 <ul>
#                   <li>orderTotal | The total value of the order
#                     <ul>
#                       <li>amount | The order amount | number</li>
#                       <li>currency | The currency id used for the order | string</li>
#                     </ul> | ref.hash
#                   </li>
#                 </ul> | ref.hash
# return       $Status | The new status of the order reference:
#                         ORDER_DRAFT, ORDER_OPEN, ORDER_SUSPENDED, ORDER_CANCELED or ORDER_CLOSED | string
#========================================================================================
sub setOrderDetails {
    my $self = shift;
    my ($refId, $hArgs) = @_;

    my $Url = $self->_setQueryArgs($self->_orderUrl($refId));
    my $Response = $self->client->post($Url, $self->_httpHeaders(), $self->encodeRequest($hArgs));
    my $OrderRef = DE_EPAGES::AmazonPayments::API::DTO::OrderReferenceInfo->new($self->decodeResponse($Response->content));

    return $OrderRef->transStatus;
}

#========================================================================================
# function     confirmOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Status = $Client->confirmOrder($refId);
#----------------------------------------------------------------------------------------
# description  Confirm the order and put it in Status ORDER_OPEN
#----------------------------------------------------------------------------------------
# input        $refId | Amazon order reference id | string
#               $SuccessURL | URL for SCA success callback | string
#               $FailureURL | URL for SCA failure callback | string
# return       Throws ConfirmOrderFailed if order could not be confirmed (e.g. client
#               needs to correct information first).
#               Throws ConfirmOrderInvalid if order could not be found or has expired.
#========================================================================================
sub confirmOrder {
    my $self = shift;
    my ($refId, $SuccessURL, $FailureURL) = @_;

    my $Url = $self->_setQueryArgs(
            $self->_orderConfirmationUrl($refId),
            { 'successUrl' => $SuccessURL, 'failureUrl' => $FailureURL}
    );

    $self->client->post($Url, $self->_httpHeaders());
    return;
}

#========================================================================================
# function     closeOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client->closeOrder($orderRefId);
#----------------------------------------------------------------------------------------
# description  Close an order
#----------------------------------------------------------------------------------------
# input        $refId | Amazon oder reference id | string
# return
#========================================================================================
sub closeOrder {
    my $self = shift;
    my ($refId) = @_;

    my $Url = $self->_setQueryArgs($self->_closeOrderUrl($refId));
    $self->client->post($Url, $self->_httpHeaders());
    return;
}

#========================================================================================
# function     cancelOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client->cancelOrder($orderRefId);
#----------------------------------------------------------------------------------------
# description  Cancels an order
#----------------------------------------------------------------------------------------
# input        $refId | Amazon oder reference id | string
# return
#========================================================================================
sub cancelOrder {
    my $self = shift;
    my ($refId) = @_;

    my $Url = $self->_setQueryArgs($self->_cancelOrderUrl($refId));
    $self->client->post($Url, $self->_httpHeaders());
    return;
}

#========================================================================================
# function     authorize
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Status = $Client->authorize($refId, $authRefId, $amount, $currency);
#----------------------------------------------------------------------------------------
# description  Request authorization to capture money for an order
#----------------------------------------------------------------------------------------
# input        $refId | Order reference id | string
# input        $authRefId | User defined reference id, up to 32 characters | string
# input        $amount | The amount to request authorization for | number
# input        $currency | The currency id to use | string
# return       $AuthorizationDetails | DE_EPAGES::AmazonPayments::API::DTO::AuthorizationDetails | object
#========================================================================================
sub authorize {
    my $self = shift;
    my ($refId, $authRefId, $amount, $currency) = @_;

    my $Url = $self->_setQueryArgs($self->_authorizeUrl);
    my $Response;

    $Response = $self->client->post($Url, $self->_httpHeaders(), $self->encodeRequest({
            orderReferenceId         => $refId,
            authorizationReferenceId => $authRefId,
            authorizationAmount      => {
                currency => $currency,
                amount   => $amount
            },
        })
    );

    return DE_EPAGES::AmazonPayments::API::DTO::AuthorizationDetails->new($self->decodeResponse($Response->content));
}

#========================================================================================
# function     getAuthorizationDetails
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Status = $Client->getAuthorizationDetails($authId);
#----------------------------------------------------------------------------------------
# description  Request information about an Amazon capture authorization
#----------------------------------------------------------------------------------------
# input        $authId | Amazon authorization id | string
# return       $AuthorizationDetails | DE_EPAGES::AmazonPayments::API::DTO::AuthorizationDetails | object
#========================================================================================
sub getAuthorizationDetails {
    my $self = shift;
    my ($authId) = @_;

    my $Url = $self->_setQueryArgs($self->_authorizationUrl($authId));
    my $Response = $self->client->get($Url, $self->_httpHeaders());
    return DE_EPAGES::AmazonPayments::API::DTO::AuthorizationDetails->new($self->decodeResponse($Response->content));
}

#========================================================================================
# function     closeAuthorization
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client->closeAuthorization($authId);
#----------------------------------------------------------------------------------------
# description  Closes an authorization. Call this when no more captures will be done
#               against the authorization
#----------------------------------------------------------------------------------------
# input        $authId | Amazon authorization id | string
# return
#========================================================================================
sub closeAuthorization {
    my $self = shift;
    my ($authId) = @_;

    my $Url = $self->_setQueryArgs($self->_closeAuthorizationUrl($authId));
    $self->client->post($Url, $self->_httpHeaders());
    return;
}

#========================================================================================
# function     capture
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CaptureDetails = $Client->capture($authId);
#----------------------------------------------------------------------------------------
# description  Capture an order amount from Amazon
#----------------------------------------------------------------------------------------
# input        $authId | The id of the open authorization | string
# input        $captureRefId | User-defined capture reference id, max. 32 characters | string
# input        $amount | Amount to capture | number
# input        $currency | Currency id to capture in | string
# return       $CaptureDetails | DE_EPAGES::AmazonPayments::API::DTO::CaptureDetails | object
#========================================================================================
sub capture {
    my $self = shift;
    my ($authId, $captureRefId, $amount, $currency) = @_;

    my $Url = $self->_setQueryArgs($self->_captureUrl);
    my $Response = $self->client->post($Url, $self->_httpHeaders(), $self->encodeRequest({
            orderAuthorizationId => $authId,
            captureReferenceId => $captureRefId,
            captureAmount      => {
                currency => $currency,
                amount   => $amount,
            },
        })
    );
    return DE_EPAGES::AmazonPayments::API::DTO::CaptureDetails->new($self->decodeResponse($Response->content));
}

#========================================================================================
# function     getCaptureDetails
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CaptureDetails = $Client->getCaptureDetails($captureId);
#----------------------------------------------------------------------------------------
# description  Get detail information about a capture
#----------------------------------------------------------------------------------------
# input        $captureId | Amazon capture id | string
# return       $CaptureDetails | DE_EPAGES::AmazonPayments::API::DTO::CaptureDetails | object
#========================================================================================
sub getCaptureDetails {
    my $self = shift;
    my ($captureId) = @_;

    my $Url = $self->_setQueryArgs($self->_captureDetailsUrl($captureId));
    my $Response = $self->client->get($Url, $self->_httpHeaders());
    return DE_EPAGES::AmazonPayments::API::DTO::CaptureDetails->new($self->decodeResponse($Response->content));
}

sub _httpHeaders {
    my $self = shift;

    my $hKeySet = $self->_keySet();
    return {
        'X-AccessKeyId'      => $hKeySet->{'AccessKeyId'},
        'X-AccessKeySecret'  => $hKeySet->{'SecretKey'},
        'X-MerchantId'       => $hKeySet->{'MerchantID'},
    };
}

sub _setQueryArgs {
    my $self = shift;
    my ($Url, $hArgs) = @_;

    $Url->query_form(
        region           => $self->_region(),
        env              => $self->_operationMode(),
        %{$hArgs // {}},
    );

    return $Url;
}

### getters ####
sub _checkUrl { return shift->{CheckUrl}; }
sub _orderUrl {
    my $self = shift;
    my ($refId) = @_;
    return $self->{OrderUrl}->uri({ refId => $refId });
}
sub _orderConfirmationUrl {
    my $self = shift;
    my ($refId) = @_;
    return $self->{OrderConfirmationUrl}->uri({ refId => $refId });
}
sub _closeOrderUrl {
    my $self = shift;
    my ($refId) = @_;
    return $self->{CloseOrderUrl}->uri({ refId => $refId });
}
sub _cancelOrderUrl {
    my $self = shift;
    my ($refId) = @_;
    return $self->{CancelOrderUrl}->uri({ refId => $refId });
}
sub _authorizationUrl {
    my $self = shift;
    my ($authId) = @_;
    return $self->{AuthorizationUrl}->uri({ authId => $authId });
}
sub _closeAuthorizationUrl {
    my $self = shift;
    my ($authId) = @_;
    return $self->{CloseAuthorizationUrl}->uri({ authId => $authId });
}
sub _authorizeUrl { return shift->{AuthorizeUrl}; }
sub _captureUrl { return shift->{CaptureUrl}; }
sub _captureDetailsUrl {
    my $self = shift;
    my ($captureId) = @_;
    return $self->{CaptureDetailsUrl}->uri({ captureId => $captureId });
}
sub _region   { return shift->{Region}; }
sub _keySet   { return shift->{hKeySet}; }
sub _operationMode { return shift->{OperationMode} }

1;