#========================================================================================
# package      DE_EPAGES::AmazonPayments::API::Object::LineItemPaymentAmazonPayments
# state        public
# base         DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal
# description  object interface for LineItemPaymentAmazonPayments.
#========================================================================================
package DE_EPAGES::AmazonPayments::API::Object::LineItemPaymentAmazonPayments;
use base DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal;

use strict;
use DE_EPAGES::AmazonPayments::API::Constants qw (
    AMAZON_AUTHORIZATION_STATUS
    AUTHORIZATION_CLOSED
    AUTHORIZATION_DECLINED
    AUTHORIZATION_OPEN
    CAPTURE_CLOSED
    CAPTURE_COMPLETED
    CAPTURE_PENDING
    ORDER_OPEN
    ORDER_CANCELED
);
use DE_EPAGES::AmazonPayments::API::Payment qw ( ConvertAmazonAddress StoreAmazonShippingAddress StoreAmazonBillingAddress );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use HTTP::Status qw ( :constants );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $LineItemPaymentAmazonPayments->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'LineItemPaymentAmazonPayments' }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentAmazonPayments = DE_EPAGES::AmazonPayments::API::Object::LineItemPaymentAmazonPayments->insert($hInfo);
# example      my $Class = LoadClassByAlias('LineItemPaymentAmazonPayments');
#               my $LineItemPaymentAmazonPayments = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertLineItemPaymentAmazonPayments' is triggered.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $LineItemPaymentAmazonPayments | new object | object
# hook         OBJ_InsertLineItemPaymentAmazonPayments | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentAmazonPaymentsID - line item payment id - int
#               </ul>
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertLineItemPaymentAmazonPayments',
      {'Object' => $self, 'LineItemPaymentAmazonPaymentsID' => $self->id }
    );
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentAmazonPayments->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteLineItemPaymentAmazonPayments' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteLineItemPaymentAmazonPayments | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentAmazonPaymentsID - line item payment id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteLineItemPaymentAmazonPayments',
      {'Object' => $self, 'LineItemPaymentAmazonPaymentsID' => $self->id }
    );
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentAmazonPayments->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateLineItemPaymentAmazonPayments'
#               and 'OBJ_AfterUpdateLineItemPaymentAmazonPayments'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateLineItemPaymentAmazonPayments | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentAmazonPaymentsID - line item payment id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateLineItemPaymentAmazonPayments | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentAmazonPaymentsID - line item payment id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateLineItemPaymentAmazonPayments',
      {'Object'=>$self, 'Values'=> $hValues, 'LineItemPaymentAmazonPaymentsID' => $self->id, 'LanguageID' => $LanguageID}
    );
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateLineItemPaymentAmazonPayments',
      {'Object'=>$self, 'Values'=> $hValues, 'LineItemPaymentAmazonPaymentsID' => $self->id, 'LanguageID' => $LanguageID}
    );
    return;
}

#========================================================================================
# function     confirmOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemPayment->confirmOrder();
#----------------------------------------------------------------------------------------
# description  Confirm the order and put it in Status ORDER_OPEN
#----------------------------------------------------------------------------------------
# input        $SuccessURL | URL for SCA success callback | string
#               $FailureURL | URL for SCA failure callback | string
#----------------------------------------------------------------------------------------
# return
#========================================================================================
sub confirmOrder {
    my $self = shift;
    my ($SuccessUrl, $FailureUrl) = @_;

    my $Basket = $self->parent()->parent();

    eval {
        $self->_client->confirmOrder($self->get('TransID'), $SuccessUrl, $FailureUrl);
        my $OrderDetails = $self->orderDetails;
        my $hAddress = ConvertAmazonAddress($self, $OrderDetails->address, $OrderDetails->buyerEmail);
        StoreAmazonShippingAddress($Basket, $hAddress);
        $self->set({
            TransTime  => GetCurrentDateTime()
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'REQUEST_FAILED') {
            my $statusCode = $Error->vars->{'Status'};
            if ($statusCode eq HTTP_BAD_REQUEST) {
                Error('ConfirmOrderFailed', {}, $Error);
            }
            if ($statusCode eq HTTP_NOT_FOUND or $statusCode eq HTTP_METHOD_NOT_ALLOWED) {
                Error('ConfirmOrderFailedRecoverable', {}, $Error);
            }
        }
        Error($Error);
    }
    return;
}

#========================================================================================
# function     orderDetails
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Details = $LineItemPayment->orderDetails();
#----------------------------------------------------------------------------------------
# description  Get detailed information about an Amazon Order Reference
#----------------------------------------------------------------------------------------
# return       $OrderReferenceInfo | Order reference information | DE_EPAGES::AmazonPayments::API::DTO::OrderReferenceInfo
#========================================================================================
sub orderDetails {
    my $self = shift;
    return $self->_client->getOrderDetails($self->get('TransID'));
}

#========================================================================================
# function     updateOrderDetails
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemPayment->updateOrderDetails();
#----------------------------------------------------------------------------------------
# description  Updates amount and currency on amazon.
#========================================================================================
sub updateOrderDetails {
    my $self = shift;
    $self->_client->setOrderDetails($self->get('TransID'), {
        'orderTotal' => {
            'amount'   => $self->container->get('GrandTotal'),
            'currency' => $self->container->get('CurrencyID'),
        }
    });
    return;
}

#========================================================================================
# function     authorize
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemPayment->authorize;
#----------------------------------------------------------------------------------------
# description  Starts the authorization for the order synchronously and triggers
#               'API_UpdateAmazonPaymentsAuthorization' for updating the
#               status of the transaction.
#========================================================================================
sub authorize {
    my $self = shift;

    # only authorize once unless it was declined previously.
    if ( $self->get('AuthCode') && ($self->get('TransStatus') ne AUTHORIZATION_DECLINED )) {
        Error('AlreadyAuthorized', {
            LineItem => $self->pathString,
            AuthCode => $self->get('AuthCode')
        });
    }

    my $Container = $self->container;
    $self->_withErrorHandling(sub {
        my $Authorization = $self->_client->authorize(
            $self->get('TransID'),
            DE_EPAGES::AmazonPayments::API::DTO::AuthorizationDetails->guidToReferenceId($self->_newAuthRef->guid),
            $Container->get('GrandTotal'),
            $Container->get('CurrencyID')
        );

        $self->set({
                CurrencyID   => $Authorization->authorizationAmount->currency,
                Amount       => $Authorization->authorizationAmount->amount,
                AuthCode     => $Authorization->authorizationId,
                TransTime    => GetCurrentDateTime(),
        });

        my $BillingAddress = $Authorization->billingAddress;
        if(defined($BillingAddress)) {
            my $Basket = $self->parent->parent;
            my $ShippingAddress = $Basket->get('ShippingAddress');
            StoreAmazonBillingAddress($Basket, ConvertAmazonAddress($self, $BillingAddress, $ShippingAddress ? $ShippingAddress->get('EMail') : undef));
        }

        TriggerHook('API_UpdateAmazonPaymentsAuthorization', {
            'LineItemPayment'      => $self,
            'AuthorizationDetails' => $Authorization,
        });
    });
    return;
}

#========================================================================================
# function     authorizationDetails
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Details = $LineItemPayment->authorizationDetails();
#----------------------------------------------------------------------------------------
# description  Request information about an Amazon capture authorization
#----------------------------------------------------------------------------------------
# return       $AuthorizationDetails | DE_EPAGES::AmazonPayments::API::DTO::AuthorizationDetails | object
#========================================================================================
sub authorizationDetails {
    my $self = shift;
    return $self->_client->getAuthorizationDetails($self->get('AuthCode'));
}

#========================================================================================
# function     capture
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemPayment->capture();
#----------------------------------------------------------------------------------------
# description  Captures the order amount from Amazon, and triggers
#               'API_UpdateAmazonPaymentsCapture' for updating the transaction status
#========================================================================================
sub capture {
    my $self = shift;

    my $Container = $self->container;
    $self->_withErrorHandling(sub {
        my $CaptureDetails = $self->_client->capture(
            $self->get('AuthCode'),
            DE_EPAGES::AmazonPayments::API::DTO::CaptureDetails->guidToReferenceId($self->_newCaptureRef->guid),
            $Container->get('GrandTotal'),
            $Container->get('CurrencyID')
        );

        $self->set({
            TransID2 => $CaptureDetails->captureId,
        });

        TriggerHook('API_UpdateAmazonPaymentsCapture', {
            'LineItemPayment' => $self,
            'CaptureDetails' => $CaptureDetails
        });
    });

    return;
}

#========================================================================================
# function     closeAuthorization
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemPayment->closeAuthorization();
#----------------------------------------------------------------------------------------
# description  Close the authorization reference
#----------------------------------------------------------------------------------------
# return
#========================================================================================
sub closeAuthorization() {
    my $self = shift;

    $self->_withErrorHandling(sub { $self->_client->closeAuthorization($self->get('AuthCode')) });
    return;
}

#========================================================================================
# function     closeOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemPayment->closeOrder();
#----------------------------------------------------------------------------------------
# description  Close the order reference
#----------------------------------------------------------------------------------------
# return
#========================================================================================
sub closeOrder() {
    my $self = shift;

    $self->_withErrorHandling(sub { $self->_client->closeOrder($self->get('TransID')) });
    return;
}

#========================================================================================
# function     cancelOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemPayment->cancelOrder();
#----------------------------------------------------------------------------------------
# description  Close the order reference
#----------------------------------------------------------------------------------------
# return
#========================================================================================
sub cancelOrder() {
    my $self = shift;

    # cancel order on amazon if needed.
    $self->_withErrorHandling(sub {
        if($self->orderDetails->transStatus eq ORDER_OPEN) {
            $self->_client->cancelOrder($self->get('TransID'));
            $self->set({
                TransStatus  => ORDER_CANCELED,
                TransTimeMod => GetCurrentDateTime(),
            });
        }
    });

    return;
}

#========================================================================================
# function     reauthorizeAndCapture
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemPayment->reauthorizeAndCapture();
#----------------------------------------------------------------------------------------
# description  Implements a flow to capture and reauthorize if neccessary.
#========================================================================================
sub reauthorizeAndCapture {
    my $self = shift;
    my $Order = $self->parent->parent;

    # shortcut: don't do anything if order is already paid.
    return if ($Order->get('PaidOn'));

    my $TransStatus = $self->get('TransStatus');

    # nothing to do if capture is already triggered.
    return if($TransStatus ~~ [CAPTURE_PENDING, CAPTURE_COMPLETED, CAPTURE_CLOSED]);
    my $DoReauthorize = ($TransStatus ~~ [AUTHORIZATION_CLOSED, AUTHORIZATION_DECLINED]);
    my $DoCapture = 0;

    $self->_withErrorHandling(sub {
        # if AUTHORIZATION_OPEN we do an additional realtime check if the authorization has not expired.
        if ($TransStatus eq AUTHORIZATION_OPEN) {
            my $AuthStatus = $self->authorizationDetails->authorizationStatus;
            $DoReauthorize = $AuthStatus eq AMAZON_AUTHORIZATION_STATUS->{(AUTHORIZATION_CLOSED)};
            $DoCapture     = $AuthStatus eq AMAZON_AUTHORIZATION_STATUS->{(AUTHORIZATION_OPEN)};
        }

        # try to issue another authorization....
        if($DoReauthorize) {
            # We can only reauthorize if the order reference is in Open state
            my $OrderStatus = $self->orderDetails->transStatus;
            if($OrderStatus eq ORDER_OPEN) {
                # unset the old authorization to be able to reauthorize.
                $self->set({
                    AuthCode => undef,
                 });
                # TODO: might want to use captureNow once available
                $self->authorize();
                $DoCapture = 1;
            }
            return;
        }

        if ($DoCapture) {
            $self->capture();
        } else {
            GetLog('USERFAULT::AmazonPayments')->warn( sprintf('Cannot capture (invalid state). ShopID:%s, ShopAlias:%s, Order:%s, OrderRef:%s',
                $Order->getSite->id,
                $Order->getSite->alias,
                $Order->pathString,
                $self->get('TransID')
            ));
        }
    });

    return;
}

sub _newAuthRef {
    my $self = shift;
    my $Folder = $self->folder('References');
    return $Folder->insertChild('AmazonPaymentsAuthorizationRef', {
        'Alias' => $Folder->nextAlias('AuthorizationRef_%d')
    });
}

sub _newCaptureRef {
    my $self = shift;
    my $Folder = $self->folder('References');
    return $Folder->insertChild('AmazonPaymentsCaptureRef', {
        'Alias' => $Folder->nextAlias('CaptureRef_%d'),
    });
}

sub _withErrorHandling {
    my $self = shift;
    my ($sub) = @_;
    $self->set({Message => undef});
    eval {
        $sub->();
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'REQUEST_FAILED' && $Error->message =~ /"message":"(.*?)(, Request|")/) {
            $self->set({Message => $1});
            Warning($Error, { ShortenedMessage => $1 });
        } else {
            Error($Error);
        }
    }
}

sub _client {
    my $self = shift;
    return $self->get('PaymentMethod')->client;
}

1;
