#==============================================================================
# package      DE_EPAGES::AmazonPayments::UI::OrderProcess
# state        public
# description  UI class for Basket Order Process
#==============================================================================
package DE_EPAGES::AmazonPayments::UI::OrderProcess;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

use base DE_EPAGES::Order::UI::OrderProcess;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogDebug LogError );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::Order::UI::Basket_Ext;

use strict;

sub ViewMultiCheckoutAmazonAddress {
    my $self = shift;
    my ($Servlet) = @_;

    $self->_registerSession($Servlet);

    # enable execution of template logic even though our 
    # ViewAction is not listed in Order/Templates/SF/SF-MultiCheckoutStep-Content.html
    $Servlet->vars('UseBaseBasketForm', 1);
    TriggerHook('UI_ViewMultiCheckoutAmazonAddress', {'Servlet' => $Servlet});
    $self->SUPER::ViewMultiCheckoutAddress($Servlet);

    # trigger hook
    TriggerHook('UI_AfterViewMultiCheckoutAmazonAddress', {'Servlet' => $Servlet});
    return;
}

sub ViewMultiCheckoutAmazonPayment {
    my $self = shift;
    my ($Servlet) = @_;

    # enable execution of template logic even though our
    # ViewAction is not listed in Order/Templates/SF/SF-MultiCheckoutStep-Content.html
    $Servlet->vars('UseBaseBasketForm', 1);
    TriggerHook('UI_ViewMultiCheckoutAmazonPayment', {'Servlet' => $Servlet});
    $self->SUPER::ViewMultiCheckoutPayment($Servlet);

    # trigger hook
    TriggerHook('UI_AfterViewMultiCheckoutAmazonPayment', {'Servlet' => $Servlet});
    return;
}

sub SaveMultiCheckoutAmazonAddress {
    my $self = shift;
    my ($Servlet) = @_;

    TriggerHook('UI_SaveMultiCheckoutAmazonAddress', {'Servlet' => $Servlet});
    $self->SUPER::Save($Servlet);
    TriggerHook('UI_AfterSaveMultiCheckoutAmazonAddress', {'Servlet' => $Servlet});

    $self->CreateOrder($Servlet,'AmazonAddress');
    return;
}

sub SaveMultiCheckoutAmazonPayment {
    my $self = shift;
    my ($Servlet) = @_;

    # do amazon validations.
    TriggerHook('UI_SaveMultiCheckoutAmazonPayment', {'Servlet' => $Servlet});
    $self->SUPER::Save($Servlet);
    TriggerHook('UI_AfterSaveMultiCheckoutAmazonPayment', {'Servlet' => $Servlet});

    # hide the address display on the confirmation page, we replace them with
    # Amazon widgets in read-only mode
    $Servlet->vars('HideConfirmationAddress', 1);
    $self->CreateOrder($Servlet,'AmazonPayment');

    return;
}

sub OrderConfirmationAmazonPayment {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;
    my $Container = $Basket->container // return;

    my $Payment = $Container->get('Payment');
    return unless ($Payment && $Payment->instanceOf('LineItemPaymentAmazonPayments'));

    LogDebug('OrderConfirmationAmazonPayment', $Payment);

    my $SuccessUrl = $self->_getSuccessUrl($Servlet, $Basket);
    my $FailureUrl = $self->_getFailureUrl($Servlet, $Basket);

    eval {
        $Payment->confirmOrder($SuccessUrl, $FailureUrl);
    };
    if (ExistsError()) {
            my $Error = GetError();
            if ($Error->code() eq 'ConfirmOrderFailedRecoverable') {
                _CheckAllConstraints($Payment, $Servlet);
             }else {
                _handleErrors($Servlet);
            }
    }

    return;
}

sub _getSuccessUrl {
    my $self = shift;
    my ($Servlet, $Basket) = @_;

    my $hSuccessUrlParams = {
        'ChangeAction' => 'SaveMultiCheckoutConfirmation',
    };
    my $hCallbackUrlOptions = {
        'Type' => 'sf/mobile',
        'AddToken' => '1',
        'UseSSL' => '1',
    };
    my $SuccessUrl = $self->_getCallbackShopUrl($Basket, $hSuccessUrlParams, $hCallbackUrlOptions, $Servlet);

    return $SuccessUrl;
}

sub _getFailureUrl {
    my $self = shift;
    my ($Servlet, $Basket) = @_;

    my $hUrlParams = {
        'ViewAction' => 'ViewMultiCheckoutAmazonPayment',
    };
    my $hCallbackUrlOptions = {
        'Type' => 'sf/mobile',
        'AddToken' => '1',
        'UseSSL' => '1',
    };
    my $FailureUrl = $self->_getCallbackShopUrl($Basket, $hUrlParams, $hCallbackUrlOptions, $Servlet);

    return $FailureUrl;
}

sub _getCallbackShopUrl {
    my $self = shift;
    my ($Object, $hArgs, $hOptions, $Servlet) = @_;

    return BuildShopUrl($Object, $hArgs, $hOptions, $Servlet);
}

sub _registerSession {
    my $self = shift;
    my ($Servlet) = @_;

    DE_EPAGES::Order::UI::Basket_Ext->RegisterSession($Servlet);

    return;
}

sub _CheckAllConstraints {
    my ($PaymentLineItem, $Servlet) = @_;
    my $aConstraints = $PaymentLineItem->orderDetails->constraints;
    if (scalar @$aConstraints) {
        if('ShippingAddressNotSet' ~~ $aConstraints) {
            $Servlet->vars('ViewAction', 'ViewMultiCheckoutAmazonAddress');
            $Servlet->form->executeFormError({
                'Name'   => 'ShippingAddressNotSet',
                'Reason' => 'ShippingAddressNotSet',
            });
        }
        if('PaymentPlanNotSet' ~~ $aConstraints) {
            $Servlet->vars('ViewAction', 'ViewMultiCheckoutAmazonPayment');
            $Servlet->form->executeFormError({
                'Name'   => 'PaymentPlanNotSet',
                'Reason' => 'PaymentPlanNotSet',
            });
        }
        if('PaymentMethodNotAllowed' ~~ $aConstraints) {
            $Servlet->vars('ViewAction', 'ViewMultiCheckoutAmazonPayment');
            $Servlet->form->executeFormError({
                'Name'   => 'PaymentMethodNotAllowed',
                'Reason' => 'PaymentMethodNotAllowed',
            });
        }
        if('AmountNotSet' ~~ $aConstraints) {
            $PaymentLineItem->updateOrderDetails;
        }
    }
}

sub _handleErrors {
    my ($Servlet) = @_;
    my $Error = GetError();
    LogError($Error);

    if($Error->code() eq 'InvalidCountry') {
        $Servlet->vars( 'ViewAction','ViewMultiCheckoutAmazonAddress');
        $Servlet->form()->executeFormError({ 'Name'   => 'UnsupportedShippingAddress', 'Reason' => 'UnsupportedShippingAddress' });

    } else {
        Error('INVALID_FORM', {
            'ViewVars'=> { 'FormError' => 0, 'ViewObjectID' => $Servlet->object()->id(), 'AmazonPaymentsError' => 1 },
            'ViewAction' => 'ViewMultiCheckoutBasket'
        });
    }
    return;
}

1;
