#========================================================================================
# package      DE_EPAGES::AppStore::UI::AppStore
# state        private
#========================================================================================
package DE_EPAGES::AppStore::UI::AppStore;
use base DE_EPAGES::Presentation::UI::Object;

use DE_EPAGES::AppStore::API::Object::AppsClient;
use DE_EPAGES::AppStore::API::Object::AppsPager;
use DE_EPAGES::AppStore::API::Url qw ( BuildRedirectUrl );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Core::API::Url qw ( UrlEncodeValue ); ## no critic (RequireCorrectImports)
use DE_EPAGES::Core::API::PerlTools qw (ElementOf ); 
use HTTP::Status;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::AppStore::API::Constants qw( APPSTORE_APP_TYPES THEMESTORE_APP_TYPES );
use feature 'switch';
use strict;
use Try::Tiny;

sub ViewAppStore {
    my $self = shift;
    my ($Servlet) = @_;

    my $categoryId = $Servlet->params('categoryId');
    # set selected category for template if there is any.
    $Servlet->vars( 'SelectedCategory', $categoryId );

    my $AppsClient = $self->_createClient($Servlet);
    my $Pager      = DE_EPAGES::AppStore::API::Object::AppsPager->new(
        { Servlet => $Servlet } );
    # set additional query params to pager links
    if(defined $categoryId){
        my $hUriParams = {'categoryId' => $categoryId};
        $Pager->{'Attributes'}->{'UriParameters'} = $hUriParams;
    }

    my $Apps = $AppsClient->listApps( $Servlet->params('Page'), APPSTORE_APP_TYPES, $categoryId );
    $Pager->handle( $Apps );

    # set all categories list from the response for template.
    $Servlet->vars( 'AllCategories', $Apps->{'facets'}->{'categories'});
    return $Pager->page;
}

sub UnityViewAppStore {
    my $self = shift;
    my ($Servlet) = @_;

    my $AppsClient = $self->_createClient($Servlet);

    my $hParams = {};
    $hParams->{'resultsPerPage'} = 1000;

    my $Apps = $AppsClient->listApps( 1, APPSTORE_APP_TYPES, undef, undef, $hParams);
    my $aCategories = $Apps->{'facets'}->{'categories'};
    
    my $ahCategoryApps = [];
    foreach my $CategoryName (@$aCategories){
        my $hCategoryWithApps = {};   
        $hCategoryWithApps->{'CategoryName'} = $CategoryName;
        foreach my $hApp (@{$Apps->{'content'}}){
            push (@{$hCategoryWithApps->{'Apps'}}, $hApp) if ElementOf($CategoryName,$hApp->{'collections'});
        }
        $hCategoryWithApps->{'AppsCount'} = scalar(@{$hCategoryWithApps->{'Apps'}});
        push @{$ahCategoryApps}, $hCategoryWithApps; 
    }
    
    $Servlet->vars( 'CategoryApps', $ahCategoryApps);
}

sub ViewThemeStore {
    my $self = shift;
    my ($Servlet) = @_;

    my $AppsClient = $self->_createClient($Servlet);
    my $Pager      = DE_EPAGES::AppStore::API::Object::AppsPager->new(
        { Servlet => $Servlet } );
    my $Themes = $AppsClient->listApps( $Servlet->params('Page'), THEMESTORE_APP_TYPES );
    $Pager->handle( $Themes );
    return $Pager->page;
}


sub ViewMyApps {
    my $self = shift;
    my ($Servlet) = @_;

    my $AppsClient = $self->_createClient($Servlet);
    my $myApps     = $AppsClient->myApps( $Servlet->params('Page') );
    $Servlet->vars( 'Apps', $myApps );
    return;
}

sub ViewAppDetails {
    my $self = shift;
    my ($Servlet) = @_;

    my $appID = $Servlet->params('appID');
    TestString( 'appID', $appID );

    my $AppsClient  = $self->_createClient($Servlet);

    try{
        my $SelectedApp = $AppsClient->getApp($appID);
        my $AppLinks    = $AppsClient->getAppLinks($appID);
        TriggerHook( 'UI_CustomizeAppLinks',
            {
                'App'      => $SelectedApp,
                'AppLinks' => $AppLinks,
                'Shop'     => $Servlet->object
            }
        );
        _prepareLinks( $SelectedApp, $AppLinks, $Servlet->object );

        $Servlet->vars( 'epagesjError', '');
        $Servlet->vars( 'App',      $SelectedApp );
        $Servlet->vars( 'AppLinks', $AppLinks );

    } catch {
        #TODO send error messages to template based on received error code from epagesj
        $Servlet->vars( 'epagesjError', 'true');
        return;
    };

    return;
}

sub _prepareLinks {
    my ( $SelectedApp, $AppLinks, $Shop ) = @_;
    my $sfUrl = BuildShopUrl( $Shop , {} , {'Type'   => 'sf', 'UseObjectPath' => '1'});
    my $adminUrl = BuildShopUrl( $Shop , {} , {'Type'   => 'admin', 'UseSSL'=>'1'});

    foreach my $link ( @{$AppLinks->{'links'}} ) {
        given ( $link->{'rel'} ) {
            when ('openApp') {
                # $openAppUrl= {adminUrlAction}&shopUrl={shopUrl}
                # =>
                # https://hostname/epages/DemoShop.admin?objectID=10000&shopUrl=<ENCODED_SF_URL>
                my $openAppUrl   = $link->{'href'};
                $openAppUrl =~ s/\{adminUrlAction\}/$adminUrl/e;
                $openAppUrl =~ s/\{shopUrl\}/UrlEncodeValue($sfUrl)/e;
                $link->{'href'} = $openAppUrl;
            }
        }
    }

    if ( $SelectedApp->{'appType'} eq 'THEME' || $SelectedApp->{'appType'} eq 'AFFILIATE' ) {
        foreach my $link ( @{$AppLinks->{'links'}} ) {
            given ( $link->{'rel'} ) {
                when ('installTheme') {
                    my $themeUrl   = $link->{'href'};
                    my $System     = LoadRootObject();
                    my $ProviderId = $System->get('ProviderID');
                    $themeUrl =~ s/\{providerID\}/UrlEncodeValue($ProviderId)/e;
                    $themeUrl =~ s/\{shopUrl\}/UrlEncodeValue($sfUrl)/e;
                    $link->{'href'} = $themeUrl;
                }
            }
        }
    }

    return;
}

sub ViewAppConsent {
    my $self = shift;
    my ($Servlet) = @_;

    my $appID = $Servlet->params('appID');
    TestString( 'appID', $appID );

    my $AppsClient  = $self->_createClient($Servlet);
    my $SelectedApp = $AppsClient->getApp($appID);
    $Servlet->vars( 'App', $SelectedApp );
    return;
}

sub SubmitAppConsent {
    my $self = shift;
    my ($Servlet) = @_;

    my $appID = $Servlet->params('appID');
    TestString( 'appID', $appID );
    my $scope = $Servlet->params('scope');
    my $dev = $Servlet->params('devApp');

    my $hParams = {};

    if ($Servlet->params('InstantInstallation')){
        $hParams->{'ViewAction'} = $Servlet->params('RedirectViewAction');
        $hParams->{'ViewObject'} = $Servlet->object->getSite->loadObject($Servlet->params('RedirectViewObjectID'),'Object' );
        $hParams->{'InstantInstallation'} = 1;
    }

    my $AppsClient     = $self->_createClient($Servlet);
    my $PendingRequest = $AppsClient->submitAppConsent($appID,$scope);

    my $RedirectUrl = BuildRedirectUrl( $Servlet, $PendingRequest->{callbackUrl}, $appID, $dev, $hParams );

    my $Response = $Servlet->response;
    $Response->code(RC_SEE_OTHER);
    $Response->header( 'Location' => $RedirectUrl->as_string );
    $Servlet->vars( 'ExitAfterEvent', 1 );

    return;
}

sub ViewUninstallAppPage {
    my $self = shift;
    my ($Servlet) = @_;

    my $appID = $Servlet->params('appID');
    TestString( 'appID', $appID );

    my $AppsClient  = $self->_createClient($Servlet);
    my $SelectedApp = $AppsClient->getApp($appID);
    $Servlet->vars( 'App', $SelectedApp );
    return;
}

sub UninstallApp {
    my $self = shift;
    my ($Servlet) = @_;

    my $appID = $Servlet->params('appID');
    TestString( 'appID', $appID );

    my $AppsClient = $self->_createClient($Servlet);
    $AppsClient->uninstall($appID);
    return;
}

sub _createClient {
    my $self = shift;
    my ($Servlet) = @_;

    return DE_EPAGES::AppStore::API::Object::AppsClient->create(
        {
            Shop   => $Servlet->object,
            Locale => $Servlet->locale
        }
    );
}

1;
