#==============================================================================
# package      DE_EPAGES::Core::API::String
# state        public
#----------------------------------------------------------------------------------------
# description  String manipulation functions
#==============================================================================
package DE_EPAGES::Core::API::String;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    QuoteTags
    QuoteXML
    UnquoteXML
    EncodeStructure
    DecodeStructure
    EncodeNonISO1Chars
    SplitEMailAddresses
    Slice
    Substr
    Decode
    ConvertNewlines
    CompareStrings
    GetWordCount
    CheckForNonPrintableChars
);

use Encode qw( decode encode );
use DE_EPAGES::Core::API::Error qw( Error QuoteStringForError );
use DE_EPAGES::Core::API::PerlTools qw ( unique UsePackage );
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::Constants qw( DEFAULT_STRINGCOMPARE_COVERAGE );

#========================================================================================
sub ErrorDefinition {
    return {
        'FALSE_URL_ENCODING'  => 'False encoded value "#Value" (encoding: #Encoding).',
        'NON_PRINTABLE_CHARS' => 'Non printable characters detected (#Chars) in string "#String"'
    };
}
#========================================================================================

#========================================================================================
# function     CheckForNonPrintableChars
# state        public
#----------------------------------------------------------------------------------------
# syntax       $String = CheckForNonPrintableChars($String);
# example      CheckForNonPrintableChars($TestString);
#----------------------------------------------------------------------------------------
# description  Checks if the given string contains non-printable chars like NUL, BEL, ...
#               If found, the NON_PRINTABLE_CHARS error is thrown.
#----------------------------------------------------------------------------------------
# input        $String | input string | string
#========================================================================================
sub CheckForNonPrintableChars {
    my ($String) = @_;

    my ($CleanString, $aFoundChars) = QuoteStringForError($String);

    if (scalar @$aFoundChars > 0) {
        Error('NON_PRINTABLE_CHARS', {
            'String'             => $CleanString,
            'Chars'              => join(', ', @$aFoundChars)
        });
    }
}

#========================================================================================
# function     QuoteTags
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Output = QuoteTags( $Input );
# example      print QuoteTags( "<bigger>&</bigger>" );
#----------------------------------------------------------------------------------------
# description  Replaces the special characters < and > by &lt; and &gt; respectively.
#----------------------------------------------------------------------------------------
# input        $Input | input string | string
# return       $Output | output string | string
#========================================================================================
sub QuoteTags($) { ## no critic qw(RequireArgUnpacking RequireCommentForExportedFunctions)
    local $_ = $_[0];

    s/</&lt;/g;
    s/>/&gt;/g;

    return $_;
}

#========================================================================================
# function     QuoteXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Output = QuoteXML( $Input );
# example      print QuoteXML( "<bigger>&</bigger>" );
#----------------------------------------------------------------------------------------
# description  Replaces the special characters &"'<> by their corresponding XML entities.
#----------------------------------------------------------------------------------------
# input        $Input | input string | string
# return       $Output | output string | string
#========================================================================================
sub QuoteXML($) { ## no critic qw(RequireArgUnpacking RequireCommentForExportedFunctions)
    local $_ = $_[0];

    return undef unless defined $_;

    s/&/&amp;/g;
    s/"/&quot;/g; # "
    s/'/&apos;/g; # '
    s/</&lt;/g;
    s/>/&gt;/g;

    return $_;
}

#========================================================================================
# function     UnquoteXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Output = UnquoteXML( $Input );
#----------------------------------------------------------------------------------------
# description  Replaces standard XML entities by the corresponding special characters.
#----------------------------------------------------------------------------------------
# input        $Input | input string | string
# return       $Output | output string | string
#========================================================================================
sub UnquoteXML($) { ## no critic qw(RequireArgUnpacking RequireCommentForExportedFunctions)
    local $_ = $_[0];

    return undef unless defined $_;

    s/&quot;/"/g; # "
    s/&apos;/'/g; # '
    s/&lt;/</g;
    s/&gt;/>/g;
    s/&amp;/&/g;

    return $_;
}

sub EncodeStructure; # forward declaration
#========================================================================================
# function     EncodeStructure
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EncodedStructure = EncodeStructure( $Encoding, $Structure )
# example      $EncodedStructure = EncodeStructure( 'iso-8859-1', \%Structure );
#----------------------------------------------------------------------------------------
# description  Encodes all strings in a nestes array/hash structure with a given
#               character encoding. The function creates a copy of the structure. The
#               original structure is not changed.
#----------------------------------------------------------------------------------------
# input        $Encoding | character encoding, example: 'iso-8859-1' | string
# input        $Structure | a tree structure of arrays and/or hashes | ref
# return       $EncodedStructure | the structure with encoded strings | ref
#========================================================================================
sub EncodeStructure {
    my ($Encoding, $Structure) = @_;

    return undef unless defined $Structure;
    return encode( $Encoding, $Structure ) unless ref $Structure;
    return [ map { EncodeStructure( $Encoding, $_ ) } @$Structure ] if ref $Structure eq 'ARRAY';
    return { map { EncodeStructure( $Encoding, $_ ) } (%$Structure) } if ref $Structure eq 'HASH';
    die "Can't encode $Structure";
}

sub DecodeStructure ($$); # forward declaration
#========================================================================================
# function     DecodeStructure
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Structure = DecodeStructure( $Encoding, $EncodedStructure )
# example      $Structure = DecodeStructure( 'iso-8859-1', $EncodedStructure );
#----------------------------------------------------------------------------------------
# description  Decodes all strings in a nestes array/hash structure with a given
#               character encoding. The function creates a copy of the structure. The
#               original structure is not changed.
#----------------------------------------------------------------------------------------
# input        $Encoding | character encoding, example: 'iso-8859-1' | string
# input        $EncodedStructure | the structure with encoded strings | ref
# return       $Structure | a tree structure of arrays and/or hashes | ref
#========================================================================================
sub DecodeStructure ($$) {
    my ($Encoding, $Structure) = @_;

    return undef unless defined $Structure;
    return decode( $Encoding, $Structure ) unless ref $Structure;
    return [ map { DecodeStructure( $Encoding, $_ ) } @$Structure ] if ref $Structure eq 'ARRAY';
    return { map { DecodeStructure( $Encoding, $_ ) } (%$Structure) } if ref $Structure eq 'HASH';

    # Can't decode Structure
    return $Structure;
}

#========================================================================================
# function     EncodeNonISO1Chars
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Output   = EncodeNonISO1Chars( $Input );
# example      "&#8364;" = EncodeNonISO1Chars( "" );
#----------------------------------------------------------------------------------------
# description  Replaces the characters outside of ISO1-range with Unicode in XML.
#----------------------------------------------------------------------------------------
# input        $Input | input string | string
# return       $Output | output string | string
#========================================================================================
sub EncodeNonISO1Chars {
    my ($string) = @_;
    return unless defined $string;
#    my $i;
#    my $result ='';
#    for ($i=0; $i<length($string); $i++) {
#        my $Char = substr($string,$i,1);
#        my $CharCode = ord($Char);
#        if ( $CharCode > 255 ) {
#            $result.=sprintf("&#%d;", $CharCode);
#        }  else {
#            $result.=$Char;
#        }
#    }
#    return $result;

    return join ('', map{   my $CharCode = ord($_);
                            if ( $CharCode > 255 ) { ## no critic (ProhibitMagicNumbers)
                                sprintf('&#%d;', $CharCode);
                            }  else {
                                $_;
                            }
                        } split (//, $string));

#    return join('', map((  ord($_) > 255 ? sprintf("&#%4d;", $CharCode) :
#             ( ord($_) >= 161 ? $ISO2HTML[ord($_) - 161] : $_ )), split(//, $word)));


}

#========================================================================================
# function     SplitEMailAddresses
# state        private
#----------------------------------------------------------------------------------------
# syntax       $aAddresses = SplitEMailAddresses($Adresses);
#----------------------------------------------------------------------------------------
# example      $aAddresses = SplitEMailAddresses('"Manuel Blechschmidt" <mblechschmidt@epages.de>; "Thomas; \"Mller" <tmueller@epages.de>"');
#----------------------------------------------------------------------------------------
# description  Split the addresses on plain /[,;]/ (surrounded by \s). Plain means
#               not in " ... " or ( ... ) or < ... >.
#----------------------------------------------------------------------------------------
# input        $Adresses | string | multiple emails, alias and quota possible
# return       $aAddresses | list of e-mail recipients, for example:
#                       [ "'\"Manuel Blechschmidt\" <mblechschmidt@epages.de>",
#                         "\"Thomas; Mller\" <tmueller@epages.de>"
#                       ] | ref.array.string
#========================================================================================
sub SplitEMailAddresses($) {
    my ($AddrString) = @_;

    my (@AddrList, $AddressPushed);
    my ($InQuote, $InComment, $InAngle, $Escaped) = (0, 0, 0, 0);
    my $Address = '';

    foreach my $str (split (//,$AddrString)) {
        $AddressPushed = 0;
        if ($InAngle) {
            $InAngle++ if $str eq '<' and not $Escaped;
            $InAngle-- if $str eq '>' and not $Escaped;
        } elsif ($InComment) {
            $InComment++ if $str eq '(' and not $Escaped;
            $InComment-- if $str eq ')' and not $Escaped;
        } elsif ($str eq '"' and not $Escaped) {
            $InQuote = not $InQuote;
        } elsif ($str eq '<' and not $InQuote and not $Escaped) {
            $InAngle = 1;
        } elsif ($str eq '(' and not $InQuote and not $Escaped) {
            $InComment = 1;
        } elsif ($str =~ /[,;]/ and not $InQuote) {
            $Address =~ s/^\s+//;
            $Address =~ s/\s+$//;
            push @AddrList, $Address if $Address =~ /\S/;
            $Address = '';
            $AddressPushed = 1;
        }
        $Address .= $str unless $AddressPushed;
        $Escaped = not $Escaped if $str =~ m/\\/;
        $Escaped = 0 unless $str =~ m/\\/;
    }
    $Address =~ s/^\s+//;
    $Address =~ s/\s+$//;
    push @AddrList, $Address if $Address =~ /\S/;
    return \@AddrList;
}

#========================================================================================
# function     Slice
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = Slice($String, $Length, $PostOrPrefix);
#----------------------------------------------------------------------------------------
# example      $String = Slice('VeryLongLongString', 10); # VeryLon...
# example      $String = Slice('VeryLongLongString', -10); # ...gString
#----------------------------------------------------------------------------------------
# description  Slice the input $String to a lenght of $Length.
#               $Length > 0: for strings longer than length, add the postfix.
#               $Length < 0: for strings longer than length, add the prefix.
#               Length of return string uncludes the $PostOrPrefix.
#----------------------------------------------------------------------------------------
# input        $String | string to slice | string
# input        $Length | length of new value, default is 30 | integer (optional)
# input        $PostOrPrefix | postfix or prefix, default is '...' | string (optional)
# return       $Value | sliced string with given length | string
#========================================================================================
sub Slice($;$$) {
    my ($String, $Length, $PostOrPrefix) = @_;
    $Length = 30 unless defined $Length; ## no critic qw(ProhibitMagicNumbers)
    $PostOrPrefix = '...' unless defined $PostOrPrefix;

    my $StringLength = length $String;
    my $AbsLength = $Length < 0 ? -$Length : $Length;
    return $String if $StringLength < $AbsLength;
    if ($Length < 0) {
        # start of new Position 70 + (- 60) + 3 = 10 + 3 = 13
        # length of new string 3 + (70 - 13) = 3 + 57 = 60
        return $PostOrPrefix . substr($String, $StringLength + $Length + length $PostOrPrefix);
    }
    return substr($String,0, $Length - length $PostOrPrefix) . $PostOrPrefix;
}

#========================================================================================
# function     Substr
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Value = Substr($String, $Start, $Length);
#----------------------------------------------------------------------------------------
# example      $Value = Substr('VeryLongLongString', 0, 8); # VeryLong
# example      $Value = Substr('VeryLongLongString', 8); # LongString
# example      $Value = Substr('VeryLongLongString', -10); # LongString
#----------------------------------------------------------------------------------------
# description  Returns a substring of $String. Use perl function substr.
#               For negative startposition the counting starts at the end of string.
#----------------------------------------------------------------------------------------
# input        $String | string | string
# input        $Start | start position for substring | integer (optional)
# input        $Length | length of substr | integer (optional)
# return       $Value | substring of input | string
#========================================================================================
sub Substr($;$$) {
    my ($String, $Start, $Length) = @_;
    return substr($String, $Start, $Length) if defined $Length;
    return substr($String, $Start);
}

#========================================================================================
# function     Decode
# state        private
#----------------------------------------------------------------------------------------
# syntax       $StringValue = Decode($Encoding, $BinValue)
#----------------------------------------------------------------------------------------
# description  decodes a value using $Encoding
#----------------------------------------------------------------------------------------
# input        $Encoding | encoding |string
# input        $BinValue | encoded value | string
# return       $StringValue | decoded string | string
#========================================================================================
sub Decode {
    my ($Encoding, $EncodedValue, $hOptions) = @_;

    return undef unless defined $EncodedValue;
    return '' if $EncodedValue eq '';

    # decode() returns undef on error with FB_QUIET
    my $StringValue = decode($Encoding, $EncodedValue, Encode::FB_QUIET);
    Error('FALSE_URL_ENCODING', {
        'Value'         => $EncodedValue,
        'Encoding'      => $Encoding
    }) unless defined $StringValue;

    # $EncodedValue contains the rest of the string that was not decoded
    # try to decode the rest with iso-1
    if ($EncodedValue ne '') {
        my $FalseValue = decode('iso-8859-1', $EncodedValue, Encode::FB_QUIET);
        Error('FALSE_URL_ENCODING', {
            'Value'         => "$StringValue$FalseValue$EncodedValue",
            'Encoding'      => 'iso-8859-1'
        }) if $EncodedValue ne '';
        $StringValue .= $FalseValue;
    }

    CheckForNonPrintableChars($StringValue);

    return $StringValue;
}

#========================================================================================
# function     ConvertNewlines
# state        Public
#----------------------------------------------------------------------------------------
# syntax       $OneLineString = ConvertNewlines($StringWithNewlines)
#----------------------------------------------------------------------------------------
# description  replace white spaces including newlines with a single space.
#----------------------------------------------------------------------------------------
# input        $Value | string containing new lines | string
# return       $Value | one-lined string  | string
#========================================================================================
sub ConvertNewlines {
    my ($Value) = @_;
    $Value =~ s/\s+/ /g; # replace multiple white space with single space
    $Value =~ s/\s+$//;  # replace trailing whitepsace
    $Value =~ s/^\s+//;  # replace leading whitepsace
    return $Value;
}

#========================================================================================
# function     CompareStrings
# state        Public
#----------------------------------------------------------------------------------------
# syntax       $Percentage = CompareStrings($StringA, $StringB)
#----------------------------------------------------------------------------------------
# description  Compare two given strings and return how much they are similar. Uses
#               word based levenshtein comparison ratio.
#               The $Coverage parameter defines how similar two words have to be to be
#               rated as equal or as different. The Default value is 0.8 (80%).
#               The returned $Percentage value is a float number between 0 and 1.0 which
#               tells how similar the both supplied strings are.
#----------------------------------------------------------------------------------------
# input        $StringA | first string | string
# input        $StringB | second string | string
# input        $Coverage | Coverage value for similarity check [0..1]. Default: 0.8 (optional) | float
# return       $Percentage | ratio how similar the strings are [0..1] | float
#========================================================================================
sub CompareStrings {
    my ( $StringA, $StringB, $Coverage ) = @_;

    if (! defined $StringA || ! defined $StringB) {
        $StringA //= '';
        $StringB //= '';
        GetLog()->info("Comparison with an undefined string");
    }

    return 1 if $StringA eq '' && $StringB eq '';
    return 0 if $StringA eq '' || $StringB eq '';

    $StringA = lc($StringA);
    $StringB = lc($StringB);

    my @Token   = _CompareStringsSplit($StringA);
    my $hStats = {};     #hash of [$TokenString, $CountHowOftenMatched]

    for ( my $i = 0; $i < scalar @Token; $i++ ) {
        $hStats->{ $Token[$i] } = [ $Token[$i], 0 ];
    }

    my @Token2 = _CompareStringsSplit($StringB);

    # find matching tokens
    FRAGMENT: foreach my $fragment ( keys %$hStats ) {
        foreach (@Token2) {
            if ( _CompareStringsIsSimilar( $fragment, $_, $Coverage ) ) {
                $hStats->{$fragment}->[1]++;
                next FRAGMENT;
            }
        }
    }

    my @Matched = grep {$hStats->{$_}->[1] > 0 } keys %$hStats;
    return scalar @Matched / scalar keys %$hStats;
}

sub _CompareStringsSplit {
    my ($String) = @_;

    return
        grep {m/[[:alnum:]]+/}
        map { s/[[:punct:]]//g; $_; }
        unique( split( /[[:blank:]]+/, $String ) );
}

sub _CompareStringsIsSimilar {
    # instead of: use Text::LevenshteinXS qw( distance );
    # because this dependency would break installCPAN.pl
    UsePackage('Text::LevenshteinXS');
    Text::LevenshteinXS->import('distance');

    my ($WordA, $WordB, $Coverage) = @_;
    # like python's Levenshtein ratio, see http://code.google.com/p/pylevenshtein/source/browse/trunk/Levenshtein.c
    my $LenSum = length($WordA) + length($WordB);
    my $Match = ($LenSum - distance($WordA, $WordB)) / $LenSum;
    $Coverage = DEFAULT_STRINGCOMPARE_COVERAGE if not defined $Coverage;
    return $Match >= $Coverage ? 1 : 0;
}

#========================================================================================
# function     GetWordCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Count = GetWordCount($String)
#----------------------------------------------------------------------------------------
# description  Returns the number of words found in a given string.
#----------------------------------------------------------------------------------------
# input        $String | text string | string
# return       $Count | number of words | integer
#========================================================================================
sub GetWordCount {
    my ($String) = @_;
    return 0 if not defined $String;
    my $Count=0;
    $Count++ while $String =~ /[\w-]+/sg;
    return $Count;
}

1;
