#==============================================================================
# package      DE_EPAGES::Customer::UI::User
# state        private
#==============================================================================
package DE_EPAGES::Customer::UI::User;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Address::API::Address qw( CheckZipcode );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::Core::API::Log qw(LogError);
use DE_EPAGES::Core::API::Password qw (CheckPassword);
use DE_EPAGES::Core::API::PerlTools qw ( CreateSecureCode );
use DE_EPAGES::Core::API::Scramble qw ( Scramble );
use DE_EPAGES::Core::API::Url qw ( BuildParameterString );
use DE_EPAGES::Customer::API::Shop qw ( GetCustomerForUser );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadRootObject Transaction );
use DE_EPAGES::Object::API::Table::Country qw ( GetInfoCountry );
use DE_EPAGES::Presentation::API::User qw( GetUsersByEMail );
use DE_EPAGES::Shop::API::Address qw ( CheckStringVATID CheckVATID UpdateVATID );
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );
use DE_EPAGES::Trigger::API::Trigger qw( TriggerHook );

#==============================================================================
# Event handlers
#==============================================================================
sub ViewPersonalData {
    my $self = shift;
    my ($Servlet) = @_;

    my $User = $Servlet->object;
    my $Shop = $User->getSite();
    my $Customer = GetCustomerForUser($User, $Shop);
    $Servlet->vars('Customer',$Customer->tleHash);
    return;
}

sub attributeValues {
    my $self = shift;
    my ( $Servlet, $Object, $Class, $Prefix ) = @_;

    my $hhValues = $self->SUPER::attributeValues($Servlet, $Object, $Class, $Prefix);

    # test valid EU-VATID
    my $Form = $Servlet->form;
    my $Shop = $Object->getSite();
    my $CountryID = $hhValues->{'Values'}->{'CountryID'};

    my $VATID = $hhValues->{'Values'}->{'VATID'};

    if ($VATID) {
        if (!CheckStringVATID($VATID)) {
            $Form->addFormError({
                'Reason'=> 'VATIDIsNotValid',
                'Form'  => 'SaveRegistration',
                'Name'  => 'VATID',
            });
        }
        else {
            $VATID = UpdateVATID( $VATID, $CountryID, $Shop );
            my $check = CheckVATID( $VATID, $CountryID, $Shop );
            $Servlet->vars('VATID', $VATID);
            if( 1 == $check ) {
                $hhValues->{'Values'}->{'VATID'} = $VATID;
            }
            else {
                $Form->addFormError({
                    'Reason'=> $check ? 'VATIDIsNotFromSameCountry' : 'VATIDIsNotValid',
                    'Form'  => 'SaveRegistration',
                    'Name'  => 'VATID',
                });
            }
        }
    }

    my $Zipcode = $hhValues->{'Values'}->{'Zipcode'};
    $Form->addFormError({
        'Reason'=> 'ZIPCODE_NOT_VALID',
        'Form'  => 'SaveRegistration',
        'Name'  => 'Zipcode',
    }) unless CheckZipcode($Zipcode, $CountryID);

    TriggerHook('UI_UserAddressValidation', {'Servlet' => $Servlet});

    return $hhValues;
}

sub SavePersonalData {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;

    my $User = $Servlet->object;
    my $OldEmail = $User->get('EMail');

    # test validity of email address if available
    $Form->form($User, 'SaveEMail');

    # get new email value and reset it in form -> do not save it
    my $NewEmail = $Form->formValue('EMail');
    my $Shop = $User->getSite();

    if ($User->get('EMail') ne $NewEmail) {
        # user must authorize email change
        my $PlainPassword = $Form->formValue('PlainPassword');
        my $StoredPasswordHash = $User->get('Password');

        unless (CheckPassword($PlainPassword, $StoredPasswordHash) ) {
            # wrong passwort we don't change anything
            $Form->executeFormError({
                'Reason'=> 'PASSWORD_NOT_CORRECT',
                'Name'  => 'Password',
                'Form'  => 'Form',
            });
        }
    }

    # test if users tries to change the email address and wants to reset to the user alias
    # or simply change to an other email address
    if ($User->get('EMail') ne $NewEmail and
        $User->get('Alias') ne $NewEmail and
        scalar @{GetUsersByEMail($NewEmail, $Shop)}
    ) {
        $Servlet->params('AddressType', '');
        $Form->executeFormError({
            'Reason'=> 'USER_EXISTS',
            'Name'  => 'EMail',
            'Form'  => 'Form'
        });
    }

    # reset EMail in form here because we do save the new email only after the mail conformation so the old email is still active
    # reset the email in form values because an other changeaction is also using the EMail field
    $Form->{'Values'}->{'EMail'}->[0] = $OldEmail;

    # save user
    $self->SUPER::Save($Servlet);

    my $Customer = GetCustomerForUser($User, $Shop);

    # email has changed or no email was there
    if (not $OldEmail or $OldEmail ne $NewEmail) {
        $Servlet->params('NewEMail', $NewEmail);
    }

    # if country is united states, state is mandatory
    my $USCountry = 0;
        if ($Form->value('CountryID')) {
        $USCountry = 1 if GetInfoCountry($Form->value('CountryID'))->{'Code2'} eq 'US';
    }

    eval {
        # check if all mandatory fields are filled in
        # test registration set
        my $aRegistrationSet = $Shop->get('RegistrationSet');
        foreach my $hField (@$aRegistrationSet) {
            my $Attribute = $hField->{'Attribute'};
            my $AttributeName = $Attribute->alias;
            my $Value = $Form->value($AttributeName);
            $hField->{'IsMandatory'} = 1 if $AttributeName eq 'State' && $USCountry;
            if ($hField->{'IsMandatory'} && (!defined $Value || $Value !~ /\S/)) {
                $Form->addFormError({
                    'Reason'=> 'STRING_EMPTY',
                    'Form'  => 'SaveRegistration',
                    'Name'  => $AttributeName,
                });
            }
        }
        my $hhCustomerAttributes = $self->attributeValues($Servlet, $Shop, $Shop->get('CustomerClass'));
        $Form->executeFormError;

        # update/insert user billing address
        my $BillingAddress = $User->get('BillingAddress');

        if (defined $BillingAddress) {
            # update address
            $Servlet->params('ObjectID', $BillingAddress->id);
            $self->SUPER::Save($Servlet);
            $self->SendUserAddressChangedMail($Servlet, $Customer, $User);
        } else {
            # insert address
            my $Parent = $User->folder('Addresses');
            my $Class = LoadClassByAlias('Address');
            Transaction( sub {
                my $_BillingAddress = $self->NewObject($Servlet, $Parent, $Class);
                $User->set({'BillingAddress' => $_BillingAddress});
            });
        }

        # set customer attributes
        $hhCustomerAttributes->{'Values'}->{'IsRatingInvitationAllowed'} = 0 unless defined $hhCustomerAttributes->{'Values'}->{'IsRatingInvitationAllowed'};
        $Customer->set( $hhCustomerAttributes->{'Values'} );
        $Customer->set( $hhCustomerAttributes->{'LocalizedValues'}{$_}, $_ ) foreach keys %{$hhCustomerAttributes->{'LocalizedValues'}};

        #START: synchronize bank account data with customer EPG-28128
        my $CustomerBillingAddress = $Customer->get('BillingAddress');
        $CustomerBillingAddress->set({
            'BankCode' => $Form->value('BankCode') // undef,
            'BankAccountHolder' => $Form->value('BankAccountHolder') // undef,
            'BankName' => $Form->value('BankName') // undef,
            'BankAccountNo' => $Form->value('BankAccountNo') // undef
        }) if $CustomerBillingAddress;
        #END: synchronize bank account data with customer EPG-28128

        $Servlet->params('ObjectID', $User->id);

        # email has changed or now email was there
        if (not $OldEmail or $OldEmail ne $NewEmail) {

            $self->SendUserEmailChangedMail($Servlet, $NewEmail);
            $User->set({
                'IsEmailVerified' => 0,
            });
            $Servlet->params('VERIFY_EMAIL_AGAIN', 1);
        }


    };
    if (ExistsError()) {
        my $Error = GetError();
        $Servlet->params('ObjectID', $User->id);
        Error($Error);
    }
    return;
}

sub SendUserAddressChangedMail ($$$) {
    my $self = shift;
    my ($Servlet, $Customer, $User) = @_;
    my $Address = $Servlet->object;


    #send user address changed mail to customer
    my $Shop = $Address->getSite();
    my $MailType = GetShopMailType( $Shop, 'UserAddressChanged', 1 );

    return unless defined $MailType;

    #Create Header
    my %MailParams = (
        'HTMLAllowed' => $User->get('IsHtmlEMailAllowed'),
    );
    my %Vars = (
        'User'      => $User->tleHash,
        'Customer'  => $Customer->tleHash,
        'Shop'      => $Shop->tleHash,
    );
    my %RegionalParams = (
        Locale      => $Servlet->params('Locale'),
        Language    => $Servlet->params('Language'),
    );

    eval {$MailType->send( $Shop, \%MailParams, \%RegionalParams, \%Vars );};

    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        $Servlet->params('SEND_MAIL_FAILED',1);
    }

    return;
}

sub SendUserEmailChangedMail {
    my $self = shift;
    my ($Servlet, $NewEmail) = @_;
    my $User = $Servlet->object;

    #send order confirmation mail to customer
    my $Shop = $User->getSite();
    my $MailType = GetShopMailType( $Shop, 'UserEmailChanged', 1 );

    return unless defined $MailType;

    #Create Header
    my %MailParams = (
        'TO'          => $NewEmail,
        'HTMLAllowed' => $User->get('IsHtmlEMailAllowed'),
    );

    my $ConfirmURL = join '/',
        $Shop->get('WebServerScriptUrlSSL'),
        $Servlet->locale,
        '?'.BuildParameterString({
            ViewObjectID => $User->id,
            ViewAction => 'ViewMyAccount',
            ChangeObjectID => $User->getSite->id,
            ChangeAction => 'ConfirmUserAccount',
            Alias => Scramble($User->alias, ''),
            EMail => Scramble($NewEmail, ''),
            NewEMail => 1,
            SecurityCode => CreateSecureCode($User->alias.$NewEmail),
            ErrorAction => 'ViewMyAccount',
            ErrorObjectID => $User->id,
        });

    my %Vars = (
        'User' => $User->tleHash,
        'Shop' => $Shop->tleHash,
        'ConfirmURL' => $ConfirmURL,
    );
    my %RegionalParams = (
        Locale      => $Servlet->params('Locale'),
        Language    => $Servlet->params('Language'),
    );

    my $From = $MailType->from;
    if( defined $From && $From =~ /\S/ ) {
        eval {$MailType->send( $Shop, \%MailParams, \%RegionalParams, \%Vars );};
        if (ExistsError()) {
            my $Error = GetError();
            LogError($Error);
            $Servlet->params('SEND_MAIL_FAILED',1);
        }
    }
    else {
        # don't log an error
        $Servlet->params('SEND_MAIL_FAILED',1);
    }

    return;
}

# Added for EPG-10712
sub SetPagerEntries {
    my $self = shift;
    my ($Servlet) = @_;
    my $User = $Servlet->object;

    my $NewPageSize = $Servlet->form->value('PageSize');
    return unless defined $NewPageSize and $NewPageSize =~ /^\d+$/;

    my $MaxPageSize = LoadRootObject()->get('MaxPageSize');
    $NewPageSize = $MaxPageSize if $NewPageSize > $MaxPageSize;

    # set the PagerEntries for a ViewAction
    my $PagerViewActionAlias = $Servlet->form->value('PagerViewAction');
    if($User->getSite()->get('IsUnityRegistered') and $PagerViewActionAlias ){
        my $ViewObjectID         = $Servlet->form->value('PagerViewObjectID');
        my $ViewObject           = $User->getSite()->loadObject($ViewObjectID,'Object');
        my $Key                  = $ViewObject->class->alias.'-'.$PagerViewActionAlias;

        # set the new value in $hPagerEntriesViewActions and trigger hook for possible changes on the hash
        my $hPagerEntriesViewActions      = $User->get('PagerEntriesViewActions');
        $hPagerEntriesViewActions->{$Key} = $NewPageSize;

        TriggerHook('UI_UserSetPagerEntries', {
                                               'Servlet'                 => $Servlet,
                                               'PagerViewActionKey'      => $Key,
                                               'PagerEntriesViewActions' => $hPagerEntriesViewActions,
                                              });

        # now set the values
        $User->set({
                    'PagerEntriesViewActions' => $hPagerEntriesViewActions,
                    'PagerEntries'            => undef,
                   });
    } else {
        # update number of pager entries
        $User->set( { 'PagerEntries' => $NewPageSize } );
    }

    return;
}

sub UnitySaveUserEmail {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my $User = $Servlet->object;
    my $Shop = $User->getSite();

    # test existence and validity of email address if available
    $Form->form($User, 'SaveUnityEMail');

    # check if mail was changed
    my $NewEmail = $Form->formValue('EMail');
    my $OldEmail = $User->get('EMail');
    return if $OldEmail eq $NewEmail;

    # test if email address is used by another user
    # test if users tries to change the email address to an existing email address
    if ( scalar @{GetUsersByEMail($NewEmail, $Shop)} ) {
        $Form->executeFormError({
            'Reason'=> 'EMAIL_EXISTS',
            'Name'  => 'EMail',
            'Form'  => 'Form'
        });
    }

    # save users email
    $self->SUPER::Save($Servlet);

    return;
}

1;
