#========================================================================================
# package      DE_EPAGES::Klarna::API::KlarnaCalc
# base         -
# state        public
# description  This class implements calculation functions for Klarna
#========================================================================================
package DE_EPAGES::Klarna::API::KlarnaCalc;
use base qw ( Exporter );
use strict;

our @EXPORT_OK = qw (
    CalcMonthlyAccountCost
    CalcMonthlyFinancingCost
    CalcTotalFinancingPurchaseCost
    Gross2net
    Net2gross
    GetEffectiveInvoiceFee
);

use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::ExternalPayment::API::Log qw (LogPayment);
use DE_EPAGES::Core::API::PerlTools qw (isValid);
use List::Util qw(max);
use Math::Round qw(nearest);
use DE_EPAGES::Klarna::API::Constants qw (
    %KLARNA_LOWESTMONTHLYPAYMENT
    KLARNA_INFINITE_UPPER_BOUND
);

use constant PERCENT => 100;
use constant MONTHS_PER_YEAR => 12;

#========================================================================================
# function        CalcMonthlyAccountCost
# state           public
#----------------------------------------------------------------------------------------
# syntax          my $result = CalcMonthlyAccountCost($hParams)
#----------------------------------------------------------------------------------------
# description     Function calculates monthly cost. Formulas are copied from php package
#                  of Klarna (monthly_cost function). To be used with Klarna Account
#----------------------------------------------------------------------------------------
# input           $hParams | - PaymentMethod  payment method object,
#                           | - Pclass  the pclass to use | object
#                           | - Basket  the basket object | object
#                             - Sum  grand total of basket | hash reference
# return          $result  | calculated monthly cost  | float
#========================================================================================
sub CalcMonthlyAccountCost {
    my ($hParams) = @_;

    my $PaymentMethod = $hParams->{'PaymentMethod'};
    Error('PaymentNotGiven') unless defined($PaymentMethod);

    my $Sum = $hParams->{'Sum'};
    Error('SumNotGiven') unless defined($Sum);

    my $Pclass = $hParams->{'Pclass'};
    Error('PclassNotGiven') unless defined($Pclass);

    my $InterestRate = $Pclass->get('InterestRate');
    my $StartFee = $Pclass->get('StartFee') * PERCENT;
    my $Fee = $Pclass->get('InvoiceFee') * PERCENT;
    my $Months = $Pclass->get('Months');
    my $Currency = $Pclass->get('CurrencyID');

    # EPG-26650: Klarna Account: calculation of monthly payment rates gives wrong results
    # this is basically the calculation code from Klarna's PHP API
    $Sum = $Sum * PERCENT + $StartFee;
    my $Annuity = _annuity($Sum, $Months, $InterestRate);
    $Annuity += $Fee;

    my $PaymentLocale = $Currency;
    # EPG-26786: Use of uninitialized value
    if($Currency eq 'EUR') {
        $PaymentLocale .= '_' . $hParams->{'Country'} unless ($hParams->{'Country'} eq '');
    }
    my $LowestMonthlyPayment = $KLARNA_LOWESTMONTHLYPAYMENT{$PaymentLocale} * PERCENT;
    my $aPaymentRates = _calculateMonthlyRates($Sum, $InterestRate, $Fee, $LowestMonthlyPayment,
        $Annuity, $Months, 1);

    # round to nearest cent. note that the Klarna documentation is wrong,
    # which says that the value should be rounded to the nearest decimal place
    my $Result = nearest('0.01', $aPaymentRates->[0] / PERCENT); # round to nearest cent

    return $Result;
}

#========================================================================================
# function        _annuity
# state           private
#----------------------------------------------------------------------------------------
# syntax          my $Annuity = _annuity($Sum, $Months, $InterestRate)
#----------------------------------------------------------------------------------------
# description     Function calculates annuity. Formulas are copied from php package
#                  of Klarna (_annuity function).
#----------------------------------------------------------------------------------------
# input           $Sum | payment sum | float
# input           $Months | number of monthly payments | int
# input           $InterestRate | interest rate | float
# return          $result  | annuity amount | float
#========================================================================================
sub _annuity {
    my ($Sum, $Months, $InterestRate) = @_;

    return $Sum if($Months == 0);
    return $Sum / $Months if($InterestRate == 0);

    my $MonthlyRate = $InterestRate / (PERCENT * MONTHS_PER_YEAR);
    return $Sum * $MonthlyRate / (1 - (1 + $MonthlyRate) ** -$Months);
}

#========================================================================================
# function        _calculateMonthlyRates
# state           private
#----------------------------------------------------------------------------------------
# syntax          my $Rates = _calculateMonthlyRates($Sum, $Rate, $Fee,
#                      $MinPay, $Annuity, $Months, $IsAccount)
#----------------------------------------------------------------------------------------
# description     Function calculates monthly payment rates. Formulas are copied from php
#                  package of Klarna (_fulpacc function).
#----------------------------------------------------------------------------------------
# input           $Sum | payment sum | float
# input           $Rate | interest rate per year | float
# input           $Fee | monthly invoice fee | float
# input           $MinPay | minimum monthly payment allowed for this country | float
# input           $Annuity | monthly payment | float
# input           $Months | number of monthly payments | int
# input           $IsAccount | use calculation for Klarna Account? | boolean
# return          $Result  | array of monthly payments | ref.array
#========================================================================================
sub _calculateMonthlyRates {
    my ($Sum, $Rate, $Fee, $MinPay, $Annuity, $Months, $IsAccount) = @_;
    my $Balance = $Sum;
    my $aResult = [];
    my $Accuracy = 1.0;

    while($Months > 0 && $Balance > $Accuracy) {
        my $Interest = $Balance * $Rate / (PERCENT * MONTHS_PER_YEAR);
        my $NewBalance = $Balance + $Interest + $Fee;
        if($MinPay >= $NewBalance || $Annuity >= $NewBalance) {
            push(@$aResult, $NewBalance);
            return $aResult;
        }

        my $NewPay = max($Annuity, $MinPay);
        $NewPay = max($NewPay, $Balance / (2 * MONTHS_PER_YEAR) + $Fee + $Interest) if($IsAccount);

        $Balance = $NewBalance - $NewPay;
        push(@$aResult, $NewPay);
        --$Months;
    }

    return $aResult;
}

#========================================================================================
# function        CalcMonthlyFinancingCost
# state           public
#----------------------------------------------------------------------------------------
# syntax          my $result = CalcMonthlyFinancingCost($hParams)
#----------------------------------------------------------------------------------------
# description     Function calculates periodic cost. Formulas are copied from php package
#                  of Klarna (periodic_cost function). To be used with Klarna Financing
#----------------------------------------------------------------------------------------
# input           $hParams | - Pclass  Pclass object,
#                             - Sum  grand total of basket | hash reference
# return          $result  | calculated periodic cost  | float
#========================================================================================
sub CalcMonthlyFinancingCost {
    my ($hParams) = @_;

    my $Pclass = $hParams->{'Pclass'};
    Error('PclassNotGiven') unless defined($Pclass);

    my $Sum = $hParams->{'Sum'};
    Error('SumNotGiven') unless defined($Sum);

    my $InterestRate = $Pclass->get('InterestRate');
    my $StartFee = $Pclass->get('StartFee') * PERCENT;
    my $Fee = $Pclass->get('InvoiceFee') * PERCENT;
    my $Months = $Pclass->get('Months');
    my $Currency = $Pclass->get('CurrencyID');
    $Sum = $Sum * PERCENT + $StartFee;
    my $Annuity = _annuity($Sum, $Months, $InterestRate);
    $Annuity += $Fee;

    my $PaymentLocale = $Currency;
    # EPG-26786: Use of uninitialized value
    $PaymentLocale .= '_' . $hParams->{'Country'} if($Currency eq 'EUR' && isValid($hParams->{'Country'}));
    my $LowestMonthlyPayment = 0; # no minimum payment for Financing
    my $aPaymentRates = _calculateMonthlyRates($Sum, $InterestRate, $Fee, $LowestMonthlyPayment,
        $Annuity, $Months, 0);

    # round to nearest cent. note that the Klarna documentation is wrong,
    # which says that the value should be rounded to the nearest decimal place
    my $Result = nearest('0.01', $aPaymentRates->[0] / PERCENT); # round to nearest cent

    return $Result;
}


#========================================================================================
# function        _totalMonthlyCost
# state           private
#----------------------------------------------------------------------------------------
# syntax          my $result = $_totalMonthlyCost($sum,$months,$monthfee,$rate,
#                                                        $startfee,$months)
#----------------------------------------------------------------------------------------
# description     Function is used when total credit purchase cost is calculated. Formulas
#                  are copied from php package of Klarna (total_monthly_cost function).
#                  Function calculates the total monthly cost (annuity payments) i.e the cost per
#                  month including all fees, the purchase sum and interest, taking account:
#                  - the startfee is paid at the first payment (with no interest)
#                  - the monthly fee is paid with every payment before any amortizing is done.
#----------------------------------------------------------------------------------------
# input           $sum | purchase sum | float
# input           $months | amount of month | integer
# input           $monthfee | monthly fee | float
# input           $rate | rate | float
# input           $startfee | start fee | float
# return          $result  | calculated total cost | float
#========================================================================================
sub _totalMonthlyCost {
    my ($sum, $months, $monthfee, $rate, $startfee) = @_;

    $monthfee = $monthfee * PERCENT;
    $startfee = $startfee * PERCENT;

    my $sum_j_to_N = 0;
    my $R = ((1.0+$rate/PERCENT)**(1.0/MONTHS_PER_YEAR));
    my $TotalCost;

    if ($rate == 0) {
        $TotalCost = (1 / $months)*($sum + ($months * $monthfee) + $startfee);
    }
    else {
        # the total cost of fees plus the cost of paying off the monthly fee first and after that amortize
        for (my $j=2; $j<$months+1; $j++) {
            $sum_j_to_N = $sum_j_to_N + ($R**($months - $j)) * $monthfee;
        }

        $TotalCost = (($R - 1) / (($R)**($months) - 1)) * (($R)**($months) * $sum + $sum_j_to_N + ($R)**($months - 1) * ($monthfee + $startfee));
    }

    $TotalCost = $TotalCost / PERCENT;
    $TotalCost = sprintf('%.2f', $TotalCost);

    return $TotalCost;
}

#========================================================================================
# function        CalcTotalFinancingPurchaseCost
# state           public
#----------------------------------------------------------------------------------------
# syntax          my $result = CalcTotalFinancingPurchaseCost($hParams)
#----------------------------------------------------------------------------------------
# description     Function calculates total credit purchase cost. Formulas are copied
#                  from php package of Klarna (total_credit_purchase_cost function).
#----------------------------------------------------------------------------------------
# input           $hParams | - Pclass: Pclass object,
#                             - Sum: grand total of basket | hash reference
# input           $PaymentTypeName | Kind of klarna payment type | String
# return          $result  | calculated periodic cost  | float
#========================================================================================
sub CalcTotalFinancingPurchaseCost {
    my ($hParams,$PaymentTypeName) = @_;

    my $Pclass = $hParams->{'Pclass'};
    Error('PclassNotGiven') unless defined($Pclass);

    my $Sum = $hParams->{'Sum'};
    Error('SumNotGiven') unless defined($Sum);

    $Sum = $Sum * PERCENT;

    my $months = $Pclass->get('Months');
    my $monthfee = $Pclass->get('InvoiceFee');
    my $startfee = $Pclass->get('StartFee');
    my $rate = $Pclass->get('InterestRate');

    my $TotalCost = _totalMonthlyCost($Sum,$months,$monthfee,$rate,$startfee);
    $TotalCost = $TotalCost * $months;
    $TotalCost = sprintf('%.0f', $TotalCost);

    LogPayment($PaymentTypeName, 'Total credit purchase cost', $TotalCost);

    return $TotalCost;
}

#========================================================================================
# function      Gross2net
# state         public
#----------------------------------------------------------------------------------------
# syntax        Gross2net($PaymentMethod, $GrossValue);
#----------------------------------------------------------------------------------------
# description   Converts a price from gross to net if necessary
#----------------------------------------------------------------------------------------
# input         $PaymentMethod | the PaymentMethod object to use | object
# input         $GrossValue | the value to convert | float
# return        $NetValue | the gross value of the input var | float
#========================================================================================
sub Gross2net {
    my ($PaymentMethod, $GrossValue) = @_;
    my $Shop = $PaymentMethod->get('Site');
    my $NetValue = $GrossValue;
    my $CurrencyID = $PaymentMethod->get('Kr_CurrencyID');

    if (defined $GrossValue && defined $CurrencyID) {
        my $TaxMatrix = $Shop->get('TaxMatrix');
        $NetValue = $TaxMatrix->gross2net(
            $CurrencyID,
            $GrossValue, {
                'TaxModel'  => $Shop->get('InputTaxModel'),
                'TaxClass'  => $TaxMatrix->defaultTaxClass,
                'Precision' => 3
            }
        );
    }

    return $NetValue;
}

#========================================================================================
# function      Net2gross
# state         public
#----------------------------------------------------------------------------------------
# syntax        Net2gross($PaymentMethod, $NetValue);
#----------------------------------------------------------------------------------------
# description   Converts a price from net to gross if necessary
#----------------------------------------------------------------------------------------
# input         $PaymentMethod | the PaymentMethod to use | object
# input         $NetValue | the value to convert | float
# return        $GrossValue | the gross value of the input var | float
#========================================================================================
sub Net2gross {
    my ($PaymentMethod, $NetValue) = @_;

    my $GrossValue = $NetValue;
    my $CurrencyID = $PaymentMethod->get('Kr_CurrencyID');

    if (defined($NetValue) && defined($CurrencyID)) {
        my $Shop = $PaymentMethod->get('Shop');
        my $TaxMatrix = $Shop->get('TaxMatrix');
        my $InputTaxModel = $Shop->get('InputTaxModel');
        $GrossValue = $TaxMatrix->net2gross(
            $CurrencyID,
            $NetValue, {
                'TaxModel' => $InputTaxModel,
                'TaxClass' => $TaxMatrix->defaultTaxClass
            },
            $InputTaxModel  ?  0 : 1
        );
    }

    return $GrossValue;
}

#========================================================================================
# function        GetEffectiveInvoiceFee
# state           private
#----------------------------------------------------------------------------------------
# syntax          my $InvoiceFee = GetEffectiveInvoiceFee($PaymentMethod, $SubTotal)
#----------------------------------------------------------------------------------------
# description     Calculate the effective invoice fee by taking care of fee boundaries
#----------------------------------------------------------------------------------------
# input           $PaymentMethod        | The payment method object  | object
# input           $SubTotal             | The total invoice sum minus costs for the payment | float
# return          $InvoiceFee           | Invoice fee | float
#========================================================================================
sub GetEffectiveInvoiceFee {
  my ($PaymentMethod, $SubTotal) = @_;

  my $InvoiceFee = $PaymentMethod->get('Kr_InvoiceFee');
  return 0 unless defined($InvoiceFee);

  if($PaymentMethod->class->alias eq 'PaymentMethodKlarnaInvoice') {
    my $InvoiceFeeLowerBound = $PaymentMethod->get('Kr_InvoiceFeeLowerBound');
    my $InvoiceFeeUpperBound = $PaymentMethod->get('Kr_InvoiceFeeUpperBound');

    $InvoiceFeeLowerBound = 0 unless defined $InvoiceFeeLowerBound; # nothing entered in MBO, assume 0
    $InvoiceFeeUpperBound = KLARNA_INFINITE_UPPER_BOUND unless defined $InvoiceFeeUpperBound; # no upper bound? assume infinity

    my $NetTotal = Gross2net($PaymentMethod, $SubTotal);

    # charge no fee if total is outside bounds (EPG-25542)
    $InvoiceFee = 0 if($NetTotal < $InvoiceFeeLowerBound || $NetTotal > $InvoiceFeeUpperBound);
  }

  return $InvoiceFee;
}