#========================================================================================
# package      DE_EPAGES::MultiStoreMaster::API::Cartridge
# base         DE_EPAGES::Shop::API::ShopInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::MultiStoreMaster::API::Cartridge;
use base DE_EPAGES::Shop::API::ShopInstaller;

use strict;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw (ExistsCartridgeByPackage);
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle RunOnStore );
use DE_EPAGES::MultiStoreCommon::API::Table::Object;
use DE_EPAGES::Object::API::Factory      qw( LoadClassByAlias LoadRootObject );

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::MultiStoreMaster::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::MultiStoreMaster.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
## no critic qw( ProhibitLongLines )
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/MultiStoreMaster',
        'Version'            => '7.38.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0'],
    );
    return bless $self, $class;
}
## use critic

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    if (ExistsCartridgeByPackage('DE_EPAGES::MultiStore')) {
        die 'You can not install MultiStore and MultiStoreMaster cartridge on the same database!';
    }
    $self->startInstall;
    $self->SUPER::install;
    $self->finishInstall;
    return;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

sub patchDBStructure_6_14_1 {
    my $self = shift;

    my $dbi = GetCurrentDBHandle();
    if($dbi->driverName() eq 'Sybase'){
        die 'MultStore migration on sybase not supported.';
    }

    $self->importPatchDir('Patch_6_14_1', 'PatchDBStructure');

    return ;
}

sub patch_6_15_4 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();
    if($dbi->driverName() ne 'mysql'){
        die 'MultiStore is supported only for MySQL.';
    }

    my $RootObject = LoadRootObject();
    $self->logInfo(<<'EOT');
-> Get values of SubShop qw(MS_WebServiceLogging MS_WebServiceURL MS_WebServiceUser MS_WebServicePassword)
   and save them locally at each MasterShop.
EOT

    $self->importPatchDir('Patch_6_15_4', 'Patch');

    my %MasterSubRefs;
    my $aAttributeNames = [ qw( MS_WebServiceLogging MS_WebServiceURL MS_WebServiceUser MS_WebServicePassword ) ];

    RunOnStore(
        'Store' => 'Site',
        'Sub' => sub {
            foreach my $ShopRefMap ( @{ LoadClassByAlias('MasterToSubShopRefMapper')->objects } ){
                my $SubShopRef      = $ShopRefMap->get('SubShopRef');
                my $MasterShopRef   = $ShopRefMap->get('MasterShopRef');
                my $MasterShopAlias = $MasterShopRef->get('StoreObjectAlias');
                $MasterSubRefs{$MasterShopAlias} = [] unless exists $MasterSubRefs{$MasterShopAlias};
                push @{$MasterSubRefs{$MasterShopAlias}}, {
                    'SubShopAlias' => $SubShopRef->get('StoreObjectAlias'),
                    'SubShopStore' => $SubShopRef->get('Store')->alias,
                };
            }
        }
    );

    my $MasterShopsFolder = LoadRootObject()->child('Shops');

    # 1. Get the old attribute values.
    my %MasterAttributes;
    my @MasterShops;
    while( my ($MasterShopAlias, $ahShopInfo) = each %MasterSubRefs ){
        next unless $MasterShopsFolder->existsChild($MasterShopAlias);
        foreach my $hInfo ( @$ahShopInfo ){
            my $SubShopAlias = $hInfo->{'SubShopAlias'};
            my $hAttributes;

            RunOnStore(
                'Store' => $hInfo->{'SubShopStore'},
                'Sub' => sub {
                    my $SubShopsFolder = $RootObject->child('Shops');
                    return unless $SubShopsFolder->existsChild($SubShopAlias);
                    $hAttributes = $SubShopsFolder->child($SubShopAlias)->get( $aAttributeNames );
                }
            );

            push @MasterShops, $MasterShopsFolder->child($MasterShopAlias);
            foreach my $AttributeName ( @$aAttributeNames ){
                $MasterAttributes{$MasterShopAlias}{$AttributeName}{$SubShopAlias} = $hAttributes->{$AttributeName};
            }

        }
    }

    # 2. Set the new attributes values.
    foreach my $MasterShop ( @MasterShops ){
        $MasterShop->set( $MasterAttributes{$MasterShop->alias} );
    }
    return '6.16.0';
}

1;
