#========================================================================================
# package      DE_EPAGES::PayPalPlus::UI::Shop
# state        public
# description  UI class
#========================================================================================
package DE_EPAGES::PayPalPlus::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::PayPalPlus::API::Constants qw (
    INTERNAL_CARTRIDGE_NAME
    MULTISTEP_CHECKOUT
);
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::PayPalPlus::API::ContextLogger;
use DE_EPAGES::PayPalPlus::API::Logger;
use DE_EPAGES::PayPalPlus::API::PaymentLineItemService;
use DE_EPAGES::PayPalPlus::API::OrderService;

use DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::Service;
use DE_EPAGES::Presentation::API::DereferrerToken qw ( ValidateDereferrerToken );
use URI;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );

use strict;

#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#----------------------------------------------------------------------------------------
# description  Name of PaymentType for search/view transactions.
#----------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName {
    return INTERNAL_CARTRIDGE_NAME;
}

#========================================================================================
# function     PaymentBacklinkPayPalPlus
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentBacklinkPayPalPlus( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction for Backlink
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub PaymentBacklinkPayPalPlus {
    my $self = shift;
    my ($Servlet) = @_;
    my $MethodName = (caller(0))[3];

    my $Context = $self->_createLogContext();
    my $Shop = $Servlet->object();

    $Context->logContext($Shop, $MethodName, sub {

        $self->BlockMonitor($Servlet);

        my $Logger = $self->_createLogger();
        $Logger->logPayment('Received Params', {'ServletParams' => $Servlet->params()});

        $self->_resetServletVariables($Servlet);
        $self->_authenticateCallback($Servlet);

        my $OrderService = $self->_createOrderService();

        if(not $OrderService->proccessOrder($Servlet->params())) {

            my $BasketOrOrder =  $OrderService->getBasketOrOrder();
            my $Reason = $OrderService->getErrorMessage();

            $self->_handleError($Servlet, $Reason, $BasketOrOrder);
        }

        $self->_showCustomerOrder($Servlet, $OrderService->getBasketOrOrder());

    });
    return;
}

#========================================================================================
# function     ViewPayPalPlusCancelPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewPayPalPlusCancelPayment( $Servlet );
#----------------------------------------------------------------------------------------
# description  ViewAction Controller for Cancel-Page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewPayPalPlusCancelPayment {
    my $self = shift;
    my ($Servlet) = @_;

    my $MethodName = (caller(0))[3];
    my $Context = $self->_createLogContext();
    my $Shop = $Servlet->object();

    $Context->logContext($Shop, $MethodName, sub {

        my $Logger = $self->_createLogger();
        $Logger->logPayment('Received Params', {'ServletParams' => $Servlet->params()});

        $self->_authenticateCallback($Servlet);

        my $PaymentLineItemService = $self->_createPaymentLineItemService();

        if($PaymentLineItemService->canLoadByGuid($Servlet->params('PaymentLineItemGUID'))) {

            my $BasketOrOrder = $PaymentLineItemService->getBasketOrOrder();

            if($self->_wasCancelled($Servlet)) {
                $PaymentLineItemService->paymentCanceled();
                $self->_handleError($Servlet, 'Canceled', $BasketOrOrder);
            } else {
                $PaymentLineItemService->paymentFailed();
                $self->_handleError($Servlet, 'Failed', $BasketOrOrder);
            }

        } else {

            $Servlet->form()->executeFormError({
                'Name'        => 'PayPalPlus_PaymentFailed',
                'Reason'      => 'BadCallback_NoPaymentGUID',
                'ViewAction'  => 'ViewPaymentPayPalPlusResult',
            });
        }

    });
}

sub _wasCancelled {
    my $self = shift;
    my ($Servlet) = @_;

    # in case of an payment error paypal appends 'ppp_msg' as a query-param to the cancel url
    return (isValid($Servlet->params('ppp_msg'))) ? 0 : 1;
}

sub _isOrderBeforePayment {
    my $self = shift;
    my ($BasketOrOrder) = @_;

    my $PaymentLineItem = $BasketOrOrder->container()->get('Payment');
    my $PaymentMethod = $PaymentLineItem->get('PaymentMethod');

    return ($PaymentMethod->get('DoPaymentOnBasket')) ? 0 : 1;
}

sub _showCustomerOrder {
    my $self = shift;
    my ($Servlet, $CustomerOrder) = @_;

    if (defined  $CustomerOrder) {
        $Servlet->vars('WasSuccessful', 1);
        $Servlet->vars('ViewObjectID', $CustomerOrder->id());

        my $UserIdentifiedPerCookie = 0;
        $Servlet->registerSession($CustomerOrder->get('User'), $UserIdentifiedPerCookie);
    }

    return;
}

sub _handleError {
    my $self = shift;
    my ($Servlet, $Reason, $BasketOrOrder) = @_;

    $self->_LogWarning('Payment Failed. Reason: '.$Reason);

    if(not defined $BasketOrOrder) {
         $Servlet->form()->executeFormError({
            'Name'        => 'PayPalPlus_PaymentFailed',
            'Reason'      => $Reason,
            'ViewAction'  => 'ViewPaymentPayPalPlusResult'
        });
    }

    $self->_restoreSession($Servlet, $BasketOrOrder);

    if(not $self->_isOrderBeforePayment($BasketOrOrder)) {
        $self->_redirectToPaymentStep($Servlet, $BasketOrOrder);
        return;
    }

    if($self->_isUnityShop($BasketOrOrder)) {
        $self->_redirectToUnity($Servlet, $BasketOrOrder, $Reason);
        return;
    }

    $Servlet->form()->executeFormError({
        'Name'        => 'PayPalPlus_PaymentFailed',
        'Reason'      => $Reason,
        'ViewAction'  => 'ViewPaymentPayPalPlusResult',
        'ObjectID'    => $BasketOrOrder->id()
    });

    return;
}

sub _restoreSession {
    my $self = shift;
    my ($Servlet, $BasketOrOrder) = @_;

    if(defined $BasketOrOrder) {
        my $UserIdentifiedPerCookie = 0;
        $Servlet->registerSession($BasketOrOrder->get('User'), $UserIdentifiedPerCookie);
    }
}

sub _redirectToUnity {
    my $self = shift;
    my ($Servlet, $BasketOrOrder, $ErrorCode) = @_;

    $ErrorCode = 'Failed' unless $ErrorCode eq 'Canceled';

    my $LineItemPayment = $BasketOrOrder->container()->get('Payment');
    $LineItemPayment->set({'PaymentErrorCode' => $ErrorCode});

    $Servlet->redirect($BasketOrOrder->get('WebUrlUnitySSL'));
    return;
}

sub _redirectToPaymentStep {
    my $self = shift;
    my ($Servlet, $Basket) = @_;

    my $Shop = $Basket->get('Shop');

    my $hParams = {};
    # in case of mobile or multistep we fallback to the default view action 'View'
    $hParams->{'ViewAction'} = 'ViewMultiCheckoutPayment' if $Shop->get('OrderProcess') == MULTISTEP_CHECKOUT and not $Servlet->vars('IsMobile');
    $hParams->{'ppp_msg'} = $Servlet->params('ppp_msg') if isValid($Servlet->params('ppp_msg'));

    my $hOptions = { 'Type' => 'sf/mobile', 'UseSSL' => 1, 'AddToken' => 1 };
    my $Url = $self->_BuildShopUrl($Basket, $hParams, $hOptions, $Servlet);

    $Servlet->redirect($Url);
    return;
}

sub _resetServletVariables {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->locale(1);
    $Servlet->languageID(1);
    $Servlet->currencyID(1);

    return;
}

sub _authenticateCallback {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $ObjectId = $Servlet->params('ObjectID');
    my $PaymentLineItemGuid = $Servlet->params('PaymentLineItemGUID');

    my $PayLoad = sprintf('%s&%s&%s', $Shop->alias(), $ObjectId, $PaymentLineItemGuid);
    my $Token = $Servlet->params('DereferrerToken');

    $self->_ValidateDereferrerToken($Token, $PayLoad);
}

sub _isUnityShop {
    my $self = shift;
    my ($BasketOrOrder) = @_;

    return  (defined $BasketOrOrder
        and defined $BasketOrOrder->get('Shop')
        and $BasketOrOrder->get('Shop')->get('IsUnityRegistered')) ? 1 : 0;
}

# The execute form error is not loggin the error that's way we have to do it this way
# Error name is the reason why it failed, unfortunally the template is not able to check
# the name, only the Reason with the execute form error
sub _LogWarning {
    my $self = shift;
    my ($Message) = @_;
    GetLog()->warn($Message);
    return;
}

sub _ValidateDereferrerToken {
    my $self = shift;
    my ($Token, $URL) = @_;
    ValidateDereferrerToken($Token, $URL);
}

sub _BuildShopUrl {
    my $self = shift;
    my ( $Object, $hParams, $hOptions, $Servlet ) = @_;
    return BuildShopUrl( $Object, $hParams, $hOptions, $Servlet );
}

sub _createPaymentLineItemService {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::PaymentLineItemService->new();
}

sub _createOrderService {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::OrderService->new();
}

sub _createLogContext {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::ContextLogger->new();
}

sub _createLogger {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Logger->new();
}

1;
