#========================================================================================
# package      DE_EPAGES::Shop::API::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the shop functions.
#========================================================================================
package DE_EPAGES::Shop::API::Shop;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    GetNextNo
    SaveShopLogo
    SynchronizeUnityShop
);

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::File qw ( DeleteFile ExistsFile fsplit MoveFile );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestString );
use DE_EPAGES::Object::API::Attributes::IncrementAttribute qw ( IncrementIntegerAttribute );
use DE_EPAGES::Object::API::Factory qw ( ExistsClassByAlias LoadClassByAlias );
use DE_EPAGES::Shop::API::Constants qw ( UNITY_LOGO );

sub ErrorDefinition {
    return {
        'NoResultsClass' => 'Class #ClassName does not exist in shop #ShopAlias',
    };
}

#========================================================================================
# function     GetNextNo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $NextNo = GetNextNo($Shop, $ClassName, $Parent, $IncNo);
#----------------------------------------------------------------------------------------
# example      $FormNo = GetNextNo($Shop, 'Invoice', $Order, 0);
#----------------------------------------------------------------------------------------
# description  Returns the next number alias for this shop.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | DE_EPAGES::Shop::API::Object::Shop
# input        $ClassName | class name | string
# input        $Parent | parent object (optional) | DE_EPAGES::Object::API::Object::Object
# input        $IncNo | increment number (optional, default: true) | boolean
# return       $NextNo | new number alias suitable for given class name | string
#========================================================================================
sub GetNextNo {
    my ($Shop, $ClassName, $Parent, $IncNo) = @_;

    $IncNo = 1 unless defined $IncNo;

    TestString('ClassName',$ClassName);
    Error('NoResultsClass', {
        'ClassName' => $ClassName,
        'ShopAlias' => $Shop->alias,
    }) unless ExistsClassByAlias($ClassName);

    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');
    TestObject('Parent',$Parent,'DE_EPAGES::Object::API::Object::Object') if $Parent;

    my $AttributeName = 'Next'.$ClassName.'No';

    #Exception: handle misnamed attribute alias
    $AttributeName = 'NextOrderNo' if $AttributeName eq 'NextCustomerOrderNo';

    my $NextNoAttribute = $Shop->class->attribute($AttributeName);
    my $Number = $Shop->get($AttributeName);

    Error('NoResultsNextNo', {
        'Attribute' => $AttributeName,
    }) if !$Number && $ClassName ne 'Product';

    my $Class;
    if (defined $Parent and $ClassName eq 'Product' and $Shop->get('IsUnityRegistered')) {
        # get the next unused number)
        while ($Parent->existsChild($Number)) {
            $Number = $Shop->formatNumber($ClassName, $Number + 1, $Parent);
        }
    }

    return $Shop->formatNumber($ClassName, $Number, $Parent) unless $IncNo;

    #Yet another silly exception for getting the right class...
    if ($ClassName eq 'Customer') {
        $Class = $Shop->get('CustomerClass');
    }
    else {
        $Class = LoadClassByAlias($ClassName);
    }

    my $NewNumber;
    do { # get the next unused number
        $Number = IncrementIntegerAttribute($Shop, $NextNoAttribute) - 1;
        $NewNumber = $Shop->formatNumber($ClassName, $Number, $Parent);
    } while scalar @{$Class->findObjects('BySiteIDClassIDAlias', [$Shop->id, $Class->id, $NewNumber])};

    # EPG-28668: MBO: Not possible to create new Customer
    # restore to previously incremented value, and increment to the next unused number
    $Number = $Shop->formatNumber($ClassName, $Number + 1, $Parent);
    while (scalar @{$Class->findObjects('BySiteIDClassIDAlias', [$Shop->id, $Class->id, $Number])}) {
        $Number = IncrementIntegerAttribute($Shop, $NextNoAttribute);
        $Number = $Shop->formatNumber($ClassName, $Number, $Parent);
    }

    return $NewNumber;
}

#========================================================================================
# function     SaveShopLogo
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $CurrentLogo = SaveShopLogo($Shop, $Form);
#----------------------------------------------------------------------------------------
# description  Saves the shop logo.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | DE_EPAGES::Shop::API::Object::Shop
# input        $Form | form object | object
#----------------------------------------------------------------------------------------
# return       $CurrentLogo | the actual shop logo path relative to the shops publicpath | string
#========================================================================================
sub SaveShopLogo {
    my ($Shop, $Form) = @_;
    my $OldShopLogo = $Shop->get('Logo');

    # Check if there is a new logo.
    my $NewLogo = $Form->value('Logo');
    if(defined $NewLogo) {
        my ($Path, undef, $Extension) = fsplit($NewLogo);
        my $hInfoShop = $Shop->get(['IsUnityRegistered', 'PublicPath']);
        if ($hInfoShop->{'IsUnityRegistered'}) {
            # rename the Logo file for Untiy shops (UNITY-4629)
            my $UploadedLogoPath = join '/', $hInfoShop->{'PublicPath'}, $NewLogo;

            $NewLogo = UNITY_LOGO . $Extension;
            my $NewLogoPath = join '/', $hInfoShop->{'PublicPath'}, $NewLogo;

            # rename uploaded logo
            MoveFile($UploadedLogoPath, $NewLogoPath);

            $Form->setValues({'Logo' => $NewLogo});
        }
        if($Path !~ m/\./ && ExistsFile(join '/', $hInfoShop->{'PublicPath'}, $NewLogo)) {
            # Delete old logo if uploaded via general settings.
            if (defined $OldShopLogo && $OldShopLogo !~ m/^MediaGallery[\/\\]/ && $OldShopLogo ne $NewLogo) {
                my $AbsoluteOldShopLogo = join '/', $hInfoShop->{'PublicPath'}, $OldShopLogo;
                DeleteFile($AbsoluteOldShopLogo) if ExistsFile($AbsoluteOldShopLogo);
            }

            # Update new Logo for Shop and Unity.
            $Shop->set({'Logo' => $NewLogo});
            return $NewLogo;
        }
    }
    return $OldShopLogo;
}

#========================================================================================
# function     SynchronizeUnityShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       SynchronizeUnityShop($Shop, $aShopAttributes);
#----------------------------------------------------------------------------------------
# description  Synchronizes all given shop attributes with the related Unity shop, which
#               are relevant for the latter.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# input        $aShopAttributes | list of shop attribute names | ref.array.string
#========================================================================================
sub SynchronizeUnityShop {
    my ($Shop, $aShopAttributes) = @_;

    # update Unity shop for each language
    my @ShopLanguageIDs = map { $_->{'LanguageID'} } @{$Shop->get('Languages')};
    foreach my $LanguageID (@ShopLanguageIDs) {
        my $hInfoShop = $Shop->get($aShopAttributes, $LanguageID);
        $Shop->updateUnityShop($hInfoShop, $LanguageID);
    }
}

1;
